﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Vita.Common {
  public class TaskErrorEventArgs : EventArgs {
    public readonly Task Task; 
    public TaskErrorEventArgs(Task task) {
      Task = task; 
    }
  }

  /// <summary>Provides centralized error logging for asynchonuous tasks.</summary>
  /// <remarks>
  /// Tasks, when fail, do not invoke Domain.UnhandledException handler, 
  /// unlike old-style methods of Thread and ThreadPool classes. So task helper provides methods for constructing Tasks attaching a continuation
  /// method that checks the result and fires the static TaskError.
  /// </remarks>
  public static class TaskHelper {

    public static event EventHandler<TaskErrorEventArgs> TaskError;

    // We might need more overloads
    public static Task NewTask(Action action) {
      var task = new Task(action);
      RegisterTask(task);
      return task; 
    }

    public static void RegisterTask(Task task) {
      task.ContinueWith(CheckTaskResult);
    }

    private static void CheckTaskResult(Task task) {
      if (task.Exception != null)
        OnTaskError(task); 
    }

    private static void OnTaskError(Task task) {
      var evt = TaskError;
      if (evt != null)
        evt(null, new TaskErrorEventArgs(task));
    }

  }
}
