﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vita.Common;

namespace Vita.Data {

  /// <summary> Extra DbType enumeration values. Should be used for DbType property of Column attribute for non-standard SQL types, like 'text', 'image' in MS SQL Server.</summary>
  public static class CustomDbTypes {
    public const DbType First = (DbType)100;

    public const DbType Text = (DbType)100;
    public const DbType NText = (DbType)101;
    public const DbType Image = (DbType)102;
    public const DbType HierarchyId = (DbType)104;
    public const DbType Geography = (DbType)105;
    public const DbType Geometry = (DbType)106;
    public const DbType SmallMoney = (DbType)107;
    public const DbType SmallDateTime = (DbType)108;
    public const DbType VarBinary = (DbType)109;
    public const DbType RowVersion = (DbType)110;

    public static bool IsCustom(DbType dbType) {
      return (int)dbType >= (int)First;
    }
    public static string AsString(DbType dbType) {
      if (!IsCustom(dbType))
        return "DbType." + dbType.ToString();
      switch (dbType) {
        case Text: return "CustomDbTypes.Text";
        case NText: return "CustomDbTypes.NText";
        case Image: return "CustomDbTypes.Image";
        case HierarchyId: return "CustomDbTypes.HierarchyId";
        case Geography: return "CustomDbTypes.Geography";
        case Geometry: return "CustomDbTypes.Geometry";
        case SmallMoney: return "CustomDbTypes.SmallMoney";
        case SmallDateTime: return "CustomDbTypes.SmallDateTime";
        case VarBinary: return "CustomDbTypes.VarBinary";
        case RowVersion: return "CustomDbTypes.RowVersion";
        default:
          Util.Throw("Invalid custom DbType value {0}", dbType);
          return null; 

      }//switch
    }//method
  }

}
