﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;

namespace Vita.Entities.Authorization {
  using Runtime;
  using Vita.Entities.Runtime; 

  public class AuthorizationException : Exception {
    public readonly Actions AttemptedAction;
    public readonly Type EntityType;
    public readonly bool IsRecordLevel;
    public readonly EntityAccessRights GrantedRights;
    public readonly string UserIdentity;
    public readonly ReadAccessLevel RequireReadMode;
    public readonly DenyReadActionType CurrentDenyReadMode; 
    public readonly string UserRoles;
    public readonly string EnabledDynamicGrants;
    public readonly string UserPermissionDetails;
    public readonly string UserContextValues;

    public AuthorizationException(string message, Type entityType, Actions attemptedAction, bool isRecordLevel, 
         EntityAccessRights grantedRights, SecureSession session) : base(message) {
      EntityType = entityType;
      AttemptedAction = attemptedAction;
      IsRecordLevel = isRecordLevel;
      GrantedRights = grantedRights;
      if (session != null) {
        RequireReadMode = session.RequireReadAccess;
        CurrentDenyReadMode = session.DenyReadAction;
        var ctx = (UserContext) session.UserContext; 
        UserIdentity = ctx.UserName;
        UserRoles = ctx.Authority.Key;
        UserContextValues = string.Join(string.Empty, ctx.Values.Select(kv => Util.SafeFormat("      [{0}]={1}\r\n", kv.Key, kv.Value)));
        UserPermissionDetails = ctx.Authority.GetPermissionsSummary(EntityType);
        var enDynGrants = ctx.Authority.DynamicGrants.Where(g => g.IsEnabled(session.UserContext));
        EnabledDynamicGrants = string.Join(",", enDynGrants.Select(g => g.Activity.Name));
          
      }
    }

    public string Summary {
      get {
        return Util.SafeFormat(
@"
    Message:             {0}
    EntityType:          {1} 
    AttemptedAction:     {2} 
    AtRecordLevel?       {3} 
    UserIdentity:        {4} 
    UserRoles:           {5}
    RequireReadAccess:   {6} 
    CurrentDenyReadMode: {7}
    Enabled dynamic grants: {8} 
    UserContext.Values: 
{11}
Granted Rights for entity: 
  {9}
========= Current user permissions for entity {1}:
  {10}.",
            Message, EntityType, AttemptedAction, IsRecordLevel, UserIdentity, UserRoles, RequireReadMode, CurrentDenyReadMode,
            EnabledDynamicGrants, GrantedRights, UserPermissionDetails, UserContextValues);
      }
    }
    public override string ToString() {
      return base.ToString() + "\r\n Authorization Information: " + Summary;
    }
  }
}
