﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Threading;
using System.Collections.Concurrent;

using Vita.Common;

namespace Vita.Entities.Authorization {

  // Tracks dynamic activity enablement in user context
  public class DynamicEnablementTable {

    #region nested class EnablementCounter
    // Need a ref wrapper for int value, to put it into the dictionary
    internal class EnablementCounter {
      public int Count;
    }
    #endregion

    private Dictionary<DynamicActivityGrant, EnablementCounter> _counters = new Dictionary<DynamicActivityGrant,EnablementCounter>();

    public void Enable(DynamicActivityGrant grant) {
      var counter = GetCounter(grant); 
      counter.Count++; 
    }

    public void Disable(DynamicActivityGrant grant) {
      var counter = GetCounter(grant);
      Util.Check(counter.Count > 0, "Invalid attempt to disable activity grant {0} - already inactive.", grant.Activity.Name);
      counter.Count--;
    }
    public bool IsEnabled(DynamicActivityGrant grant) {
      EnablementCounter counter;
      if (!_counters.TryGetValue(grant, out counter)) 
        return false;
      return counter.Count > 0;
    }

    private EnablementCounter GetCounter(DynamicActivityGrant grant) {
      EnablementCounter counter;
      if (_counters.TryGetValue(grant, out counter)) return counter; 
      counter = new EnablementCounter();
      _counters[grant] = counter;
      return counter; 
    }

  }
}
