﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;
using Vita.Entities.Model;
using Vita.Entities.Authorization.Runtime;


namespace Vita.Entities.Authorization {

  /// <summary> A container for detailed access rights information for an entity (instance or type-level) for a given user. </summary>
  public class EntityAccessRights {
    public Actions Actions;
    public EntityMemberMask PeekMask; //null is equivalent to all-bits-set
    public EntityMemberMask ReadMask;
    public EntityMemberMask UpdateMask;
    //for Type-level rights, indicates that there are record-level restrictions
    public bool HasInstanceLevelRestrictions;

    public static readonly EntityAccessRights AllowNone = new EntityAccessRights(Actions.None);
    public static readonly EntityAccessRights AllowAll = new EntityAccessRights(Actions.CRUD);

    public EntityAccessRights(Actions actions, bool hasInstanceLevelRestrictions = false, EntityMemberMask memberMask = null) {
      Actions = actions;
      HasInstanceLevelRestrictions = hasInstanceLevelRestrictions; 
      if (memberMask != null) {
        if (Actions.IsSet(Actions.Peek)) PeekMask = memberMask;
        if (Actions.IsSet(Actions.Read)) ReadMask = memberMask;
        if (Actions.IsSet(Actions.Update)) UpdateMask = memberMask;
      }
    }

    public bool CanPeek(EntityMemberInfo member) {
      return Actions.IsSet(Actions.Peek) && (PeekMask == null || PeekMask.IsSet(member));
    }
    
    public bool CanRead(EntityMemberInfo member) {
      return Actions.IsSet(Actions.Read) && (ReadMask == null || ReadMask.IsSet(member));
    }
    
    public bool CanUpdate(EntityMemberInfo member) {
      return Actions.IsSet(Actions.UpdateStrict) && (UpdateMask == null || UpdateMask.IsSet(member));
    }
    
    public bool CanUpdate() {
      return Actions.IsSet(Actions.UpdateStrict);
    }
    
    public bool CanDelete() {
      return Actions.IsSet(Actions.DeleteStrict);
    }

    public bool CanPeekAny() {
      return Actions.IsSet(Actions.Peek);
    }
    public bool CanPeekAll() {
      return Actions.IsSet(Actions.Peek) && PeekMask == null;
    }

    public bool CanReadAny() {
      return Actions.IsSet(Actions.ReadStrict);
    }
    public bool CanReadAll() {
      return Actions.IsSet(Actions.ReadStrict) && ReadMask == null;
    }

    public bool CanUpdateAny() {
      return Actions.IsSet(Actions.UpdateStrict);
    }
    public bool CanUpdateAll() {
      return Actions.IsSet(Actions.UpdateStrict) && UpdateMask == null;
    }

    public override string ToString() {
      var result = Actions.ToString();
      if (PeekMask != null || ReadMask != null || UpdateMask != null)
        result += "/Masked";
      return "(" + result + ")"; 
    }

  }//class



}
