﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;
using Vita.Entities.Model;

namespace Vita.Entities.Authorization {

  /// <summary>A permission set for an entity group. Identifies resources (entities) and allowed actions. </summary>
  public class EntityGroupPermission : Permission {
    public Actions Actions; 
    public List<EntityGroupResource> GroupResources = new List<EntityGroupResource>();

    public EntityGroupPermission(string name, Actions actions, params EntityGroupResource[] groupResources) 
      : base(name) {
      Actions = actions;
      if (groupResources != null)
        Add(groupResources);
    }

    public EntityGroupPermission(string name, Actions actions, params Type[] entityTypes) : base(name) {
      Actions = actions;
      var resource = new EntityGroupResource(name + "_resource", entityTypes);
      GroupResources.Add(resource); 
    }

    public void Add(params EntityGroupResource[] resources) {
      Util.Check(resources != null, "Parameter resources may not be null.");
      Util.CheckAllNotNull(resources, "One or more of resource objects is null.");
      GroupResources.AddRange(resources);

    }
    public override string ToString() {
      return Actions + "/" + string.Join(",", GroupResources.Select(r=>r.Name));
    }

    public override bool Allowed(IUserContext context, object resource, string operation) {
      throw new Exception(
@"EntityGroupPermission.Allowed method is not available. 
Use specialized Authorization facilities for checking access rights to entities.");
    }

    bool _initialized; //to avoid multiple initializations
    public void Init(EntityModelInfo model) {
      if (_initialized)
        return;
      _initialized = true;
      foreach (var res in this.GroupResources)
        res.Init(model);
    }

  }
}
