﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using Vita.Common;
using Vita.Entities.Model;
using Vita.Entities.Authorization.Runtime;

namespace Vita.Entities.Authorization {

  /// <summary> A container for combined permission sets assembled to match some organizational role. </summary>
  public class Role : HashedObject {
    public readonly int Id;
    public string Name;
    public List<ActivityGrant> ActivityGrants = new List<ActivityGrant>();
    public List<Role> ChildRoles = new List<Role>();

    internal RuntimeRoleData RuntimeData;
    private static int _idCounter;

    #region constructors
    public Role(string name, params Activity[] activities) : this(name, null, activities) { }

    public Role(string name, DataFilter filter, params Activity[] activities)  {
      Id = Interlocked.Increment(ref _idCounter);
      Name = name;
      Grant(filter, activities);
    }
    #endregion

    public void Grant(params Activity[] activities) {
      Grant(null, activities);
    }

    public void Grant(DataFilter filter, params Activity[] activities) {
      foreach (var act in activities)
        ActivityGrants.Add(new ActivityGrant(act, filter));
    }

    public void Grant(params Permission[] permissions) {
      Grant(null, permissions);
    }
    public void Grant(DataFilter filter, params Permission[] permissions) {
      Util.Check(permissions != null && permissions.Length > 0, "Role.Grant: Permission list is empty");
      // Create an activity
      var name = permissions[0].Name;
      if (permissions.Length > 1)
        name += "**";
      var act = new Activity(name, permissions);
      Grant(filter, act);
    }

    public DynamicActivityGrant GrantDynamic(DataFilter filter, Activity activity, string documentKey = null, DataFilter documentFilter = null) {
      var grant = new DynamicActivityGrant(activity, filter, documentKey, documentFilter);
      ActivityGrants.Add(grant);
      return grant; 
    }
    
    bool _initialized; //to avoid multiple initializations
    public void Init(EntityModelInfo model) {
      if (_initialized)
        return;
      _initialized = true;
      foreach (var role in this.ChildRoles)
        role.Init(model);
      foreach (var grant in this.ActivityGrants)
        grant.Activity.Init(model); 
    }

    public override string ToString() {
      return Name; 
    }

  }//class



}
