﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Xml;

using Vita.Entities.Runtime;

namespace Vita.Entities {
  /// <summary> An exception representing an error that does NOT manifest a system failure (no bug in software). </summary>
  /// <remarks>
  /// It might be a concurrency error (attempt to modify already changed record), not finding an entity in search, etc.
  /// A subclass of NonFatalException is ValidationException that represents a user error (invalid data submitted).
  /// </remarks>
  [Serializable]
  public class NonFatalException : Exception {
    public string ErrorCode; //one of the constants in ErrorCodes static class to identify the error type

    public NonFatalException() { } //for deserialization

    public NonFatalException(string message) : this(message, string.Empty) { }

    public NonFatalException(string message, string errorCode) : base(message) {
      ErrorCode = errorCode; 
    }
    public NonFatalException(string message, string errorCode, Exception inner) : base(message, inner) {
      ErrorCode = errorCode; 
    }

  }//class
}
