﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

using Vita.Common;
using Vita.Entities.Runtime;

namespace Vita.Entities {

  // A helper class, container for validation error (soft error).
  [Serializable]
  public class ValidationError {
    public string Code; //short tag identifying exception
    //Target object - the object that failed validation
    public string EntityName; // The entity name of the target object
    public string PrimaryKeyValue;
    public string NaturalKeyValue; //human-readable key value
    public string PropertyName; // Property name for failed object.
    public object InvalidValue;
    public int Position; // Position inside the invalid value.
    public string Message; //Localized message
    public object[] MessageArgs;
    public string CustomData;

    public ValidationError() { } //for deserialization

    public ValidationError(object invalidValue, string message, object[] messageArgs, string code, string propertyName, object entity) : 
      this(invalidValue, message, messageArgs, code, propertyName, EntityHelper.GetRecord(entity)) {}

    public ValidationError(object invalidValue, string message, object[] messageArgs, string code, string propertyName, EntityRecord record) {
      InvalidValue = invalidValue;
      Message = Util.SafeFormat(message, messageArgs);
      MessageArgs = messageArgs; 
      Code = code;
      PropertyName = propertyName;
      if (record != null) {
        EntityName = record.EntityInfo.Name;
        PrimaryKeyValue = record.PrimaryKey.ValuesToString();
      }
      //TargetNaturalKey = targetNaturalKey;
    }

    public ValidationError(object invalidValue, string message, object[] messageArgs, string code, string propertyName, string entityName, string primaryKeyValue) {
      InvalidValue = invalidValue;
      Message = Util.SafeFormat(message, messageArgs);
      MessageArgs = messageArgs;
      Code = code;
      PropertyName = propertyName;
      EntityName = entityName;
      PrimaryKeyValue = primaryKeyValue;
    }


    public ValidationError(string message, object[] messageArgs, string code, string entityName, string primaryKeyValue, string naturalKeyValue, string propertyName, object invalidValue) {
      Message = Util.SafeFormat(message, messageArgs);
      MessageArgs = messageArgs;
      Code = code;
      EntityName = entityName;
      PrimaryKeyValue = primaryKeyValue;
      NaturalKeyValue = naturalKeyValue;
      PropertyName = propertyName;
      InvalidValue = invalidValue;
    }

    
    public override string ToString() {
      return Util.SafeFormat("{0} (entity={1}, key={2}, property={3})", Message, EntityName, PrimaryKeyValue, PropertyName); 
    }
  }



}//namespace
