﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Xml;

using Vita.Entities.Runtime; 
using Vita.Common; 

namespace Vita.Entities {

  [Serializable]
  public class ValidationException : NonFatalException {
    public List<ValidationError> Errors = new List<ValidationError>();

    public ValidationException(string message) : this(message, ExceptionStrings.CodeValidationFailure) { }
    public ValidationException(string message, string errorCode) : base(message, errorCode) { }
    public ValidationException(string message, Exception inner) : base(message, ExceptionStrings.CodeValidationFailure, inner) { }

    public ValidationException(IList<ValidationError> errors) : this("Validation errors occurred") {
      Errors.AddRange(errors);  
    }

    public string[] GetMessages() {
      var list = new StringList();
      foreach (var err in Errors)
        list.Add(err.ToString());
      return list.ToArray();           
    }

    public override string ToString() {
      var nl = Environment.NewLine;
      return "ValidationException is thrown. Errors: " + nl + string.Join(nl + "  ", Errors) + nl;
    }

    public override void GetObjectData(SerializationInfo info, StreamingContext context) {
      base.GetObjectData(info, context);
    }

  }//class
}//namespace
