﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Vita.Common;
using Vita.Logging;

namespace Vita.Entities.Model.Construction {

  public static class EntityAttributeHelper {

    public static string GetUniqueKeyName(this EntityInfo entity, string defaultName) {
      var cnt = 0;
      var result = defaultName;
      while (true) {
        if(! entity.Keys.Any(k => k.Name.Equals(result, StringComparison.InvariantCultureIgnoreCase)))
          return result;
        cnt++;
        result = defaultName + cnt; 
      }
    }

    public static EntityMemberInfo FindEntityRefMember(this EntityInfo inEntity, string memberNameToFind, Type typeToFind, EntityMemberInfo listMember, Log log) {
      IList<EntityMemberInfo> refMembers;
      if (!string.IsNullOrEmpty(memberNameToFind)) {
        refMembers = inEntity.Members.FindAll(m => m.MemberName == memberNameToFind);
      } else
        refMembers = inEntity.Members.FindAll(m => m.DataType == typeToFind || m.DataType.IsSubclassOf(typeToFind));
      if (refMembers.Count == 1)
        return refMembers[0];
      //Report error
      var listMemberDesc = listMember.Entity.FullName + "." + listMember.MemberName;
      if (refMembers.Count == 0)
        log.Error("EntityList member {0}: could not find matching foreign key member in target entity {1}. ",
          listMemberDesc, inEntity.EntityType);
      if (refMembers.Count > 1)
        log.Error("EntityList member {0}: more than one matching foreign key member in target entity {1}. ",
          listMemberDesc, inEntity.EntityType);
      return null;
    }


  }

}
