﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection.Emit;
using System.Reflection;

using Vita.Common;

namespace Vita.Entities.Model.Construction {

  public class EntityClassesAssembly {

    public static object Lock = new object();
    public static EntityClassesAssembly Instance {
      get {
        if (_instance == null)
          lock (Lock) {
            if (_instance == null)
              _instance = new EntityClassesAssembly();
          }
        return _instance;

      }
    } static EntityClassesAssembly _instance;


    public AssemblyBuilder AssemblyBuilder;
    public ModuleBuilder ModuleBuilder;
    private HashSet<string> Namespaces = new HashSet<string>(); 

    public EntityClassesAssembly() {
      var assemblyName = "__Vita_EntityClasses";
      var asmName = new AssemblyName(assemblyName);
      asmName.Version = new Version(1, 0, 0, 0);
      AssemblyBuilder = System.AppDomain.CurrentDomain.DefineDynamicAssembly(asmName, AssemblyBuilderAccess.RunAndSave);
      ModuleBuilder = AssemblyBuilder.DefineDynamicModule(assemblyName);
    }

    public void AssignNamespace(EntityModelInfo modelInfo) {
      for (int i = 0; i < 10; i++) {
        var ns = "EntityClasses." + modelInfo.Model.ModelName;
        if (i > 0)
          ns += i; //append index
        modelInfo.ClassesNamespace =  ns;
        if (Namespaces.Add(ns)) //returns true if added - so it was not present before
          return; 
      }
      Util.Throw("Failed to create entity classes namespace for Model {0}: program bug or too many model instances.", modelInfo.Model.ModelName);
    }

    public void SaveAssembly() {
      AssemblyBuilder.Save(AssemblyBuilder.FullName + ".dll");
      AssemblyBuilder = null;
      ModuleBuilder = null;
    }

  
  }//class

}//ns
