﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities.Runtime;

namespace Vita.Entities.Model {

  /// <summary> Provides a way to read a value from User context. Used in authorization data filters.</summary>
  public interface IContextValueReader {
    object GetValue(IUserContext context);
  }
  public interface IContextValueReader<TValue> : IContextValueReader {
    new TValue GetValue(IUserContext context);
  }

  /// <summary>Simple implementation of IContextValueReader - reads value from UserContext.Values under a fixed key. </summary>
  public class ContextValueReader : IContextValueReader {
    public string Key;

    public ContextValueReader(string key) {
      Key = key; 
    }

    public object GetValue(IUserContext context) {
      object result; 
      context.Values.TryGetValue(Key, out result);
      return result; 
    }

  }

  public class ContextValueReader<TValue> : ContextValueReader, IContextValueReader<TValue> {

    public ContextValueReader(string key) : base(key) { }

    public new TValue GetValue(IUserContext context) {
      var ctx = (UserContext)context;
      return ctx.GetValue<TValue>(Key); 
    }
  }

}
