﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;

namespace Vita.Entities.Model {

  // Static singleton containing dictionary for looking up EntityInfo from entity type. 
  // NOTE: this is an ugly approach (singleton dict of entitytype->entityInfo), used by current deserializer(s); should be gone as soon as we switch to a new serializer

  public partial class EntityModelInfo {

    static ConcurrentDictionary<Type, EntityInfo[]> _entityInfoRegistry = new ConcurrentDictionary<Type, EntityInfo[]>();

    public static void RegisterEntity(Type type, EntityInfo entity) {
      EntityInfo[] entArray;
        if (_entityInfoRegistry.TryGetValue(type, out entArray)) {
          if (entArray.Contains(entity))
            return; //already there
          //if it is a class, make sure we have only one registration
          if (!type.IsInterface)
            Util.Throw("Cannot register multiple EntityInfo instances for entity class {0}.", type);
          Array.Resize(ref entArray, entArray.Length + 1);
          entArray[entArray.Length - 1] = entity;
          _entityInfoRegistry[type] = entArray;
        } else {
          _entityInfoRegistry[type] = new EntityInfo[] {entity};
        } //else
    }

    public static EntityInfo GetByClass(Type classType) {
      Util.Check(!classType.IsInterface, "Invalid argument: {0}, should be class type, not interface type.", classType);
      EntityInfo[] entArray;
      if (_entityInfoRegistry.TryGetValue(classType, out entArray))
        return entArray[0];
      return null;
    }

    public static EntityInfo GetByInterface(Type interfaceType, string area = null, bool firstIfMultiple = true) {
      Util.Check(interfaceType.IsInterface, "Invalid argument: {0}, should be interface type.", interfaceType);
      EntityInfo[] entArray;
      if (!_entityInfoRegistry.TryGetValue(interfaceType, out entArray))
        return null;
      if (!string.IsNullOrEmpty(area))
        return entArray.FirstOrDefault(e => e.Area.Name == area);
      if (entArray.Length == 1 || firstIfMultiple)
        return entArray[0];
      //we have multiple entities for a given type
      Util.Throw("Multiple Entities are registered for type {0}, area '{1}'", interfaceType);
      return entArray[0];
    }
  }//class

}//ns
