﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities.Model;

namespace Vita.Entities.Runtime {
  public class PropertyBoundListManyToOne<TEntity> : PropertyBoundListBase<TEntity> {

    public PropertyBoundListManyToOne(EntityRecord ownerRecord, EntityMemberInfo ownerMember) : base(ownerRecord, ownerMember) { }

    public override void Notify(BoundListEventType eventType, EntitySession session) {
      switch (eventType) {
        case BoundListEventType.SavingChanges:
          if (OwnerRecord.Status == EntityStatus.Deleting) 
            return;
          if (IsLoaded && IsOrdered && Modified)
            AssignPersistentOrder();
          break;
        case BoundListEventType.SavedChanges:
          if (session.EntityInfosForRecordsChanged.Contains(this.TargetEntity))
            Entities = null; //to force reload
          Modified = false;
          break;
        case BoundListEventType.CanceledChanges:
          Entities = null; 
          Modified = false; 
          break; 
      }//switch
    }//method

    public override void LoadList() { 
      Modified = false;
      var status = OwnerRecord.Status;
      if (status == EntityStatus.Fantom || status == EntityStatus.New) {
        Entities = new List<EntityBase>();
        return;
      }
      //check cache
      if (OwnerRecord.CachedRecord != null) {
        var cachedList = OwnerRecord.CachedRecord.GetValue(OwnerMember) as PropertyBoundListManyToOne<TEntity>;
        var cachedRecs = cachedList.GetRecords();
        var newRecs = OwnerRecord.Session.CloneAndAttach(cachedRecs);
        this.Entities = newRecs.Select(r => r.EntityInstance).ToList(); 
        return; 
      }
      // Get from database
      var recs = OwnerRecord.Session.GetChildRecords(OwnerRecord, OwnerMember.ChildListInfo.ParentRefMember);
      Entities = recs.Select(r => r.EntityInstance).ToList();
    }

    private void AssignPersistentOrder() {
      var persMember = OwnerMember.ChildListInfo.PersistentOrderMember;
      var list = Entities;
      for (int i = 0; i < list.Count; i++) {
        var rec = EntityHelper.GetRecord(list[i]);
        rec.SetValue(persMember, i + 1); // Persistent order is 1-based
      }
    }//method

  }//class
}
