﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using System.Diagnostics;

using Vita.Common;
using Vita.Entities.Model;

namespace Vita.Entities.Runtime {

  public class EntityQuery : IQueryable, IOrderedQueryable {
    public readonly EntitySession Session; 
    // If true, do not put query into cache
    public bool DisableQueryCache { get; set; }
    public object RuntimeInfo; //EntityQueryRuntimeInfo defined in Vita.Data - assigned during execution

    public EntityQuery(EntitySession session, Type elementType, Expression expression = null) {
      Session = session;
      ElementType = elementType;
      Expression = expression; 
    }

    public override string ToString() {
      return Expression.ToString();
    }

    public bool IsEntitySet {get; protected set; }


    #region IQueryable Members
    public virtual Type ElementType { get; private set; }
    public Expression Expression { get; protected set; }
    public IQueryProvider Provider { get { return Session; } }
    #endregion


    #region IEnumerable Members
    public IEnumerator GetEnumerator() {
      var iEnum = (IEnumerable) Provider.Execute(this.Expression);
      return iEnum.GetEnumerator();
    }
    #endregion

  }//class

  public class EntityQuery<TEntity> : EntityQuery, IQueryable<TEntity>, IOrderedQueryable<TEntity> {

    // Note: expression parameter is null for root entity set
    public EntityQuery(EntitySession session, Expression expression) : base(session, typeof(TEntity), expression) { }

    #region IEnumerable<TEntity> Members
    IEnumerator<TEntity> IEnumerable<TEntity>.GetEnumerator() {
      var qr = Provider.Execute<IEnumerable<TEntity>>(this.Expression);
      return  qr.GetEnumerator();
    }
    #endregion

  }//class

  public class EntitySet<TEntity> : EntityQuery<TEntity> {
    public EntitySet(EntitySession session) : base(session, null) {
      Expression = Expression.Constant(this); 
      IsEntitySet = true;
    }
    public override string ToString() {
      return "EntitySet<" + ElementType + ">";
    }

  }

}
