﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

using Vita.Common;
using Vita.Entities.Authorization;
using Vita.Entities.Model;

namespace Vita.Entities.Runtime {

  //Implementation of IEntityAccess authorization interface
  public partial class EntityRecord : IEntityAccess {

    #region IEntityAccess implementation

    public bool IsAuthorizationEnabled {
      get { return Session is SecureSession; }
    } 

    public bool CanPeek<TEntity>(Expression<Func<TEntity, object>> propertySelector) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertySelector);
      return AccessRights.CanPeek(member);
    }

    public bool CanPeek(string propertyName) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertyName, throwIfNotFound: true);
      return AccessRights.CanPeek(member);
    }

    public bool CanPeekAny() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanPeekAny();
    }

    public bool CanPeekAll() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanPeekAll();
    }

    public bool CanRead<TEntity>(Expression<Func<TEntity, object>> propertySelector) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertySelector);
      return AccessRights.CanRead(member);
    }

    public bool CanRead(string propertyName) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertyName, throwIfNotFound: true);
      return AccessRights.CanRead(member);
    }

    public bool CanReadAny() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanReadAny();
    }

    public bool CanReadAll() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanReadAll();
    }

    public bool CanUpdate<TEntity>(Expression<Func<TEntity, object>> propertySelector) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertySelector);
      return AccessRights.CanUpdate(member);
    }

    public bool CanUpdate(string propertyName) {
      if (!EnsureRecordAccessAssigned()) return true;
      var member = EntityInfo.GetMember(propertyName, throwIfNotFound: true);
      return AccessRights.CanUpdate(member);
    }
    public bool CanUpdate() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanUpdate();
    }
    public bool CanUpdateAll() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanUpdateAll();
    }

    public bool CanDelete() {
      if (!EnsureRecordAccessAssigned()) return true;
      return AccessRights.CanDelete();
    }

    //private utility method. Checks if record is secured; if yes, verifies that AccessRights information is assigned
    private bool EnsureRecordAccessAssigned() {
      if (AccessRights != null)
        return true;
      if (Session == null)
        return false;
      var secSession = Session as SecureSession;
      if (secSession != null)
        AccessRights = secSession.GetRecordAccess(this);
      else
        AccessRights = EntityAccessRights.AllowAll;
      return true;
    }


    #endregion

  }//class
}//ns
