﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;

namespace Vita.Entities.Runtime {

  //Entity store level
  public delegate void EntityStoreInitEventHandler(EntityStore entityStore, EntityStoreInitEventArgs args);
  // Entity session events; 
  public delegate void EntitySessionEventHandler(EntitySession session, EventArgs args);
  // error event
  public delegate void EntityStoreErrorEventHandler(EntitySession session, EntityStoreErrorEventArgs args);

  public enum EntityStoreInitStep {
    Constructed,
    Activating,
    EntityModelConstructed,
    DbModelConstructing, 
    DbModelConstructed, 
    DbSchemaUpdating,
    DbSchemaUpdated,
    Activated,
    Connected,
  }

  public class EntityStoreInitEventArgs : EventArgs {
    public readonly AppActivationContext ActivationContext;
    public readonly EntityStoreInitStep InitStep;
    public EntityStoreInitEventArgs(EntityStoreInitStep initStep, AppActivationContext activationContext) {
      InitStep = initStep;
      ActivationContext = activationContext;
    }
  }

  public class EntityStoreErrorEventArgs : EventArgs {
    public readonly Exception Exception;
    public EntityStoreErrorEventArgs(Exception exception) {
      Exception = exception;
    }
  }//class

  //Events for the entire entity store
  public class EntityStoreEvents {
    public event EntityStoreInitEventHandler Initializing; //fired multiple times
    //Session-associated events 
    public event EntitySessionEventHandler NewSession;
    public event EntitySessionEventHandler SavingChanges;
    public event EntitySessionEventHandler SavedChanges;
    public event EntitySessionEventHandler SaveChangesAborted;
    public event EntityStoreErrorEventHandler Error;

    internal void OnInitializing(EntityStore entityStore, EntityStoreInitStep step, AppActivationContext activationContext) {
      if (Initializing != null)
        Initializing(entityStore, new EntityStoreInitEventArgs(step, activationContext)); 
    }

    internal void OnNewSession(EntitySession entitySession) {
      if (NewSession != null)
        NewSession(entitySession, EventArgs.Empty);
    }
    internal void OnSavingChanges(EntitySession entitySession) {
      if (SavingChanges != null)
        SavingChanges(entitySession, EventArgs.Empty);
    }
    internal void OnSavedChanges(EntitySession entitySession) {
      if (SavedChanges != null)
        SavedChanges(entitySession, EventArgs.Empty);
    }
    internal void OnSaveChangesAborted(EntitySession entitySession) {
      if (SaveChangesAborted != null)
        SaveChangesAborted(entitySession, EventArgs.Empty);
    }
    internal void OnError(EntitySession entitySession, Exception exception) {
      if (Error != null)
        Error(entitySession, new EntityStoreErrorEventArgs(exception));
    }
  }//class

}//ns
