﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Concurrent;

using Vita.Common;
using Vita.Entities.Authorization;
using Vita.Logging;
using Vita.Services;

namespace Vita.Entities.Runtime {

  public class UserContext : IUserContext {

    #region IUserContext Members

    public string UserName { get; internal set;}
    public object User { get; internal set;}
    public object UserId { get; internal set;}

    public IDictionary<string, object> Values {
      get { return _values; }
    }  ConcurrentDictionary<string, object> _values = new ConcurrentDictionary<string, object>(StringComparer.InvariantCultureIgnoreCase);

    #endregion

    // Other public members which are not exposed on minimalist IUserContext interface
    public Logging.Log Log; 

    // Authority is set/used by Authorization framework
    public Authority Authority;

    // The version of the global cache that was used last time when processing user request/operation. 
    // It is used to catch the occurrence when entity cache is stale. In multi-app-server environment, when user request 
    // is routed to a server that has 'older' cache than cache that was used in previous user's operation - this is 
    // a clear indicator of a 'stale' cache on this server. The cache should be refreshed before executing the request.  
    // The last used cache version should be preserved in persistent user session between requests. 
    public long RequireCacheVersion;

    //Used by Authorization framework
    internal DynamicEnablementTable EnablementTable {
      get {
        if (_enablementTable == null)
          _enablementTable = new DynamicEnablementTable();
        return _enablementTable;
      }
    } DynamicEnablementTable _enablementTable;

    public UserContext(string userName, Authority authority = null, object userId = null, object user = null) {
      UserName = userName ?? Environment.UserName;
      Authority = authority;
      UserId = userId;
      User = user;
    }

    public override string ToString() {
      return "User:" + UserName;
    }

    public T GetValue<T>(string key) {
      object v;
      if (_values.TryGetValue(key, out v))
        return (T)v;
      return default(T);
    }

    public void RemoveValue(string key) {
      object value;
      _values.TryRemove(key, out value);
    }


  }//class

}
