﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Threading;

using Vita.Common;

namespace Vita.Logging {

  public class FileLogWriter : ILogWriter {
    string _filePath;
    object _lock = new object(); 

    public FileLogWriter(string filePath) {
      _filePath = filePath; 
      //check that dir exists and we can open the file
      var dir = Path.GetDirectoryName(filePath); 
      if (!string.IsNullOrWhiteSpace(dir))
        Util.Check(Directory.Exists(dir), "Directory '{0}' does not exist", dir);
    }

    #region ILogWriter interface
    public void Write(IList<LogEntry> entries) {
      var text = string.Join(Environment.NewLine, entries);
      WriteToFile(text);
    }
    #endregion

    private void WriteToFile(string text) {
      lock(_lock)
        WriteToFileNoLock(text);
    }

    private void WriteToFileNoLock(string text) {
      FileStream file = null;
      try {
        file = new FileStream(_filePath, FileMode.Append, FileAccess.Write);
        var writer = new StreamWriter(file);
        writer.WriteLine(text);
        writer.Flush();
      } catch (Exception ex) {
        Trace.WriteLine("\r\n Error in Log.Write: " + ex.ToString());
      } finally {
        if (file != null)
          try { file.Close(); } catch { }
      }
    }//method

  }//FileLogWriter class

}
