﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vita.Common;

namespace Vita.Logging {

  public enum LogEntryType {
    Information,
    Warning,
    Error,
  }

  //base class
  public abstract class LogEntry {
    public readonly LogEntryType EntryType;
    public LogEntry(LogEntryType entryType = LogEntryType.Information) {
      EntryType = entryType;
    }
  }

  public class MessageLogEntry : LogEntry {
    string _message;
    object[] _args;
    string _formattedMessage;
    public MessageLogEntry(LogEntryType entryType, string message, object[] args)
      : base(entryType) {
      _message = message;
      _args = args;
    }
    public override string ToString() {
      if (_formattedMessage == null)
        _formattedMessage = Util.SafeFormat(_message, _args);
      return _formattedMessage;
    }
  }//class

  public class ExceptionLogEntry : LogEntry {

    DateTime _dateTime;
    public Exception Exception;
    private string _toString;

    public ExceptionLogEntry(DateTime dateTime, Exception exception)
      : base(LogEntryType.Error) {
      _dateTime = dateTime;
      Exception = exception;
    }

    public override string ToString() {
      if (_toString == null)
        _toString = FormatException();
      return _toString;
    }

    private string FormatException() {
      return Util.SafeFormat("!!! Exception (Datetime: {0}) \r\n {1}", _dateTime, this.Exception.ToLogString());
    }

  }//class


}
