﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Vita.Entities;

namespace Vita.Services {

  /// <summary>A container for application-wide variables and settings. Use IAppContextProvider service to retrieve the context.  </summary>
  public class AppContext {
    public readonly IDictionary<string, object> Values = new Dictionary<string, object>();

    public AppContext() {
    }

    public object this[string key] {
      get {
        object result;
        if (Values.TryGetValue(key, out result))
          return result;
        return null;
      }
      set { Values[key] = value; }
    }

  }

  /// <summary>Makes app context globally available. Clean alternative to static singleton. </summary>
  public interface IAppContextProviderService {
    AppContext GetAppContext();
  }

  internal class AppContextProviderService : IAppContextProviderService {
    AppContext _context;

    public AppContextProviderService(AppContext context) {
      _context = context;
    }

    #region IAppContextProvider Members

    public AppContext GetAppContext() {
      return _context;
    }

    #endregion
  }

}
