﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Vita.Logging;
using Vita.Entities;
using Vita.Entities.Runtime;

namespace Vita.Services {

  /// <summary>An optional interface to initialize a non-module service added to Services collection of EntityModule. 
  /// Module-based services - when an EntityModule implements a service - are initialized through overridable Init method. 
  /// </summary>
  public interface IEntityServiceInit {
    /// <summary>Notifies that entity store is constructed, but not initialized yet. </summary>
    /// <param name="entityStore"></param>
    /// <remarks>This method is invoked very early in application initialization, when entity store is just constructed as an object. 
    /// To implement advanced activation at later phase (when other services are initialized), signup to entityStore.Events.Initializing event 
    /// to get notified about later phases.</remarks>
    void Init(IEntityStore entityStore);
  }

}
