﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vita.Common;

namespace Vita.Services {

  public interface IErrorLogService {
    // Note: use EntityHelper.GetUserLogContents(session) helper method to retrieve user-bound log when logging error
    bool LogError(Exception exception, string userLog, string user = null, string appName = null, string machineName = null);
    bool LogError(string message, string details, string userLog, string user = null, string appName = null, string machineName = null);
  }

  //default implementation
  /// <summary>Saves errors to Trace. </summary>
  internal class TraceErrorLogService : IErrorLogService {

    #region IErrorLogService Members

    public bool LogError(Exception exception, string localLog, string user = null, string appName = null, string machineName = null) {
      return this.LogError(exception.Message, exception.ToString(), localLog, user, appName, machineName);
    }
    public bool LogError(string message, string details, string localLog, string user = null, string appName = null, string machineName = null) {
      Trace.WriteLine("===================== ERROR ============================");
      Trace.WriteLine(Util.SafeFormat("User={0}, App={1}, Machine={2}", user, appName, machineName));
      Trace.WriteLine(message);
      Trace.WriteLine(details);
      if (!string.IsNullOrEmpty(localLog))
        Trace.WriteLine(" Local log: \r\n" + localLog);
      Trace.WriteLine("END ERROR REPORT =======================================");
      return true;

    }


    #endregion
  }

}
