﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Linq.Expressions;

using Vita.Common;
using Vita.Logging;
using Vita.Entities;
using Vita.Entities.Model;
using Vita.Entities.Runtime;

namespace Vita.Data.Caching.Implementation {
  // Used for logging queries in full set cache
  public class CacheCommandLogEntry : LogEntry {
    DateTime _dateTime;
    string _commandName;
    object[] _args; // command parameters
    long _executionTime;
    int _rowCount;
    CachingType _cacheType;
    Paging _paging; 
    private string _toString;

    public CacheCommandLogEntry(string commandName, object[] args, Paging paging, DateTime dateTime, long executionTime, int rowCount, CachingType cacheType) : base(LogEntryType.Information) {
      _commandName = commandName;
      _args = args;
      _paging = paging; 
      _dateTime = dateTime; 
      _executionTime = executionTime;
      _rowCount = rowCount;
      _cacheType = cacheType;
    }
    public override string ToString() {
      if (_toString == null)
        _toString = FormatEntry(); 
      return _toString;
    }

    private string FormatEntry() {
      var fullFormat = "/*{0}*/ {1} {2} {3} -- @{4} ms {5}, {6}";
      var strArgs = LoggingExtensions.ToSqlArgList(_args);
      var strRowCount = (_rowCount < 0) ? string.Empty : Util.SafeFormat(", {0} row(s)", _rowCount);
      var strCacheType = _cacheType == CachingType.FullSet ? "FullSetCache" : "SparseCache";
      var strPaging = _paging == null || _paging.IsNoPaging() || _cacheType == CachingType.Sparse ? string.Empty : "/*" + _paging.ToString() + "*/";
      var result = Util.SafeFormat(fullFormat, strCacheType, _commandName, strArgs, strPaging, _executionTime, strRowCount, _dateTime);
      return result;
    }


  }//DbLogEntry class

}
