﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Vita.Services;

namespace Vita.Data.Caching.Implementation {
  public partial class EntityCache {
    Task _asyncTask;

    private void ActivateAsyncWorker() {
      if (_asyncTask != null)
        return;
      //Start new async worker; do it carefully, using interlocked
      var newTask = new Task(AsyncWorker);
      var currTask = Interlocked.Exchange(ref _asyncTask, newTask);
      if (currTask != null)
        return; //already running
      _asyncTask.Start(); 
    }

    private void AsyncWorker() {
      try {
        var status = this.Status;
        if (status == CacheStatus.NonCurrent) {
          ReloadCache();
        } else
          ProcessCacheUpdateQueue(); 
      } finally {
        //clear the field, indicating that we're done
        Interlocked.Exchange(ref _asyncTask, null); 
      }
    }


  }
}
