﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Vita.Common;

namespace Vita.Data.Caching.Implementation {

  public class SimpleCacheInvalidator : ICacheInvalidator {
    GlobalEventSignaler _globalSignaler; //global invalidator - communicates with all other caches on the same machine using global event

    public SimpleCacheInvalidator(TimeSpan expirationTime) {
      _globalSignaler = new GlobalEventSignaler(this.GetType().Name, expirationTime);
      _globalSignaler.Signaled += GlobalSignaler_Signaled;  
    }

    void GlobalSignaler_Signaled(object sender, SignalEventArgs e) {
      var args = new CacheInvalidateEventArgs() { IsTimeout = e.TimedOut };
      OnInvalidated(args); 
    }

    public void OnInvalidated(CacheInvalidateEventArgs args) {
      if (Invalidated != null)
        Invalidated(this, args); 
    }

    #region ICacheInvalidator Members

    public event EventHandler<CacheInvalidateEventArgs> Invalidated;

    public void SignalInvalidate() {
      _globalSignaler.Signal(); 
    }

    #endregion
  }
}
