﻿using System;
using System.Collections.Generic;

using Vita.Common;
using Vita.Entities;
using Vita.Entities.Model;
using Vita.Data.Management;
using Vita.Data.Driver;
using System.Data;

namespace Vita.Data {

  /// <summary>Db option flags. </summary>
  [Flags]
  public enum DbOptions {
    /// <summary>Empty set.</summary>
    None = 0,

    /// <summary>Use stored procedures for CRUD access.</summary>
    UseStoredProcs = 1,

    /// <summary>Create referential integrity constraints in database. </summary>
    UseRefIntegrity = 1 << 1,

    /// <summary>Pluralize table names, ex: entity IProduct -> table Products. </summary>
    PluralizeTableNames = 1 << 2,

    /// <summary>Ignore table name case.</summary>
    IgnoreTableNamesCase = 1 << 3,

    /// <summary>Instructs system to create indexes on all foreign keys if server does not do it automatically like SQL Server.</summary>
    AutoIndexForeignKeys = 1 << 8,

    /// <summary> Default value.</summary>
    Default = UseStoredProcs | UseRefIntegrity,

    /// <summary> Default value for MS SQL Server.</summary>
    DefaultMsSql = UseStoredProcs | UseRefIntegrity | AutoIndexForeignKeys,
  }

  [Flags]
  public enum DbSchemaUpdateOptions {
    None = 0,
    VerifyKeysIndexes = 1 << 2,
    VerifyRoutines = 1 << 3,
    VerifyAll = VerifyKeysIndexes | VerifyRoutines,

    //Activation
    // BE VERY CAREFUL, we strongly discourage to ever set this flag. The recommended policy is to leave as-is the tables 
    // in database that do not match any entity in the model. 
    /// <summary> Delete all tables in schema that have no matching entities. </summary>
    DropUnknownTables = 1 << 7,

    // Actions
    UpdateSchema = 1 << 8,
    ThrowOnSchemaMismatch = 1 << 9,

    DefaultDev = UpdateSchema | VerifyAll,
    DefaultProduction = ThrowOnSchemaMismatch |  VerifyAll,
  }

}//ns
