﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Runtime.Serialization;

using Vita.Common;

namespace Vita.Data {
  /// <summary>A wrapper for runtime exception in the database. This exception is used to 'unify' different types of data exceptions for different providers.</summary>
  /// <remarks>
  /// The original database exception is saved in the Exception.Inner property. Using this exception helps making the business code to be more data-provider independent. 
  /// Note that some database exceptions might be 'switched' into NonFatalException-derived exceptions: for example, unique index violation results in database exception
  /// that is detected by the driver and is rethrown as a <c>UniqueIndexViolationException.</c>
  /// </remarks>
  public class DbException : Exception {
    public IDbCommand Command; 
    public string EntityCommandName;

    public DbException(Exception sqlException, IDbCommand command = null, string entityCommandName = null)   : base(sqlException.Message, sqlException) {
      Command = command;
      EntityCommandName = entityCommandName;
      if (!string.IsNullOrWhiteSpace(EntityCommandName))
        this.AddValue("EntityCommandName", EntityCommandName);
    }

  }//class
}//ns
