﻿using System;
using System.Data;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Collections.Generic;

using Vita.Entities;
using Vita.Data.Management;
using Vita.Data.Model;
using Vita.Entities.Runtime;

namespace Vita.Data {

  public static class DbExtensions {

    public static bool IsSet(this DbSchemaUpdateOptions options, DbSchemaUpdateOptions option) {
      return (options & option) != 0;
    }
    //Enum extensions
    public static bool IsSet(this DbOptions options, DbOptions option) {
      return (options & option) != 0;
    }

    public static void EnableCache(this IEntityStore entityStore, bool enable = true) {
      var entStore = (EntityStore)entityStore;
      var cacheControl = entStore.GetService<Vita.Data.Caching.ICacheControlService>();
      if (cacheControl != null)
        cacheControl.CacheEnabled = enable; 
    }

    public static string GetAsString(this DataRow row, string columnName) {
      var value = row[columnName];
      return value == DBNull.Value ? null : (string)value;
    }
    public static int GetAsInt(this DataRow row, string columnName) {
      var value = row[columnName];
      if (value == null || value == DBNull.Value) return 0;
      return Convert.ToInt32(value);
    }
    public static long GetAsLong(this DataRow row, string columnName) {
      var value = row[columnName];
      if (value == null || value == DBNull.Value) return 0;
      return Convert.ToInt64(value);
    }
    public static byte GetAsByte(this DataRow row, string columnName) {
      var value = row[columnName];
      if (value == null || value == DBNull.Value) return 0;
      return Convert.ToByte(value);
    }


  }//class

}//namespace
