﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Vita.Common;
using Vita.Data.Model;
using Vita.Data.Management;
using Vita.Data.Driver;

namespace Vita.Data {

  public class DbSettings {
    public readonly DbDriver Driver;
    public DbOptions Options = DbOptions.Default;
    // by default system allows 1 million records; if desired, this can be lowered or increased.
    public int QueryLimit = 1000 * 1000; //one million
    public string ConnectionString;
    public DbNamingPolicy NamingPolicy;
    public readonly StringSet Schemas = new StringSet (); //might be different from EntityModel.Areas

    //schema update settings
    public DbSchemaUpdateOptions SchemaUpdateOptions;
    public string SchemaManagementConnectionString;
    // TODO: custom schema update facility - needs more work, need to tie it to version; refactor when db/app versioning is in place
    public List<CustomDbSchemaUpdateAction> CustomUpdateActions = new List<CustomDbSchemaUpdateAction>();

    public DbSettings(DbDriver driver, DbOptions options, string connectionString, 
                      DbSchemaUpdateOptions schemaUpdateOptions = DbSchemaUpdateOptions.DefaultDev, string schemaManagementConnectionString = null, 
                      IEnumerable<string> schemas = null) {
      Driver = driver;
      Options = options;
      ConnectionString = connectionString;
      SchemaUpdateOptions = schemaUpdateOptions;
      SchemaManagementConnectionString = schemaManagementConnectionString ?? connectionString;
      NamingPolicy = new DbNamingPolicy();
      //Verify options
      if (!Driver.Supports(DbCapabilities.StoredProcedures))
        Options &= ~DbOptions.UseStoredProcs;
      if (schemas != null)
        Schemas.UnionWith(schemas); 
    }
  }//class


}
