﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Vita.Common;
using Vita.Entities.Model;
using Vita.Entities.Runtime;

namespace Vita.Data.Linq {

  /// <summary>Contains basic information about the query. Produced by the EntityQueryInfoBuilder.</summary>
  public class EntityQueryRuntimeInfo {
    public LambdaExpression Lambda;
    public HashSet<EntityInfo> Entities;
    public readonly List<object> ParamValues;
    public string CacheKey; //null if query cache should be disabled for this query

    public Vita.Data.Caching.Implementation.CompiledCacheQuery EntityCacheQuery;
    public SqlQuery SqlQuery;
    public IDbCommand DbCommand; 

    public EntityQueryRuntimeInfo(LambdaExpression lambda, HashSet<EntityInfo> entities, List<object> paramValues, string cacheKey) {
      Lambda = lambda;
      Entities = entities;
      ParamValues = paramValues;
      CacheKey = cacheKey; 
    }

    public static EntityQueryRuntimeInfo GetQueryInfo(EntityQuery query) {
      // The call may be coming from Database class, and info object might have already been constructed by CacheDataStore
      // - in this case return existing object.
      if (query.RuntimeInfo != null)
        return (EntityQueryRuntimeInfo)query.RuntimeInfo;
      var builder = new EntityQueryRuntimeInfoBuilder();
      builder.Build(query);
      return (EntityQueryRuntimeInfo)query.RuntimeInfo;
    }


  }//class


}
