﻿
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq.Expressions;
using Vita.Common;
using Vita.Data.Model;
using Vita.Entities;
using Vita.Entities.Runtime;

//RI: this class is a replacement for SelectQuery
namespace Vita.Data.Linq {

  public class SqlQueryParameter {
    public string Name;
    public Type Type;
    public Expression<Func<object[], object>> ValueReader;
    public Func<object[], object> ReadValue;

    public SqlQueryParameter(string name, Type type, Expression<Func<object[], object>> valueReader) {
      Name = name;
      Type = type;
      ValueReader = valueReader;
      ReadValue = (Func<object[], object>)valueReader.Compile();
    }
  }

  /// <summary>Represents a linq query, with SQL statement and parameter definitions. </summary>
  public partial class SqlQuery  {
      public readonly string Sql;

      /// <summary> Parameters to be sent as SQL parameters.</summary>
      public IList<SqlQueryParameter> Parameters;

      /// <summary>
      /// Expression that creates a row object
      /// Use GetRowObjectCreator() to access the object with type safety
      /// </summary>
      public readonly Func<IDataRecord, EntitySession, object> RowObjectCreator;


      /// <summary>Used on scalar calls, like First(). </summary>
      public string ExecuteMethodName { get; private set; }

      /// <summary>Indicates if DbQuery may be cached.</summary>
      public readonly bool CanCache; 


      public SqlQuery(string sql, IList<SqlQueryParameter> parameters, 
                      Func<IDataRecord, EntitySession, object> rowObjectCreator, string executeMethodName, bool canCache)     {
        Sql = sql;
        Parameters = parameters;
        RowObjectCreator = rowObjectCreator;
        ExecuteMethodName = executeMethodName;
        CanCache = canCache;
      }
    }
}
