﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Vita.Data.Model;
using Vita.Entities.Runtime;

namespace Vita.Data.Linq.Translation {

  class EntityMaterializer {
    #region OutColumMapping nested class
    public class OutColumnMapping {
      public DbColumnInfo DbColumn;
      public int ReaderColumnIndex;
    }
    #endregion

    public static MethodInfo ReadMethodInfo;
    DbTableInfo _tableInfo;
    List<OutColumnMapping> _primaryKeyColumns = new List<OutColumnMapping>();
    List<OutColumnMapping> _nonKeyColumns = new List<OutColumnMapping>();

    static EntityMaterializer() {
      ReadMethodInfo = typeof(EntityMaterializer).GetMethod("ReadEntity", BindingFlags.Instance | BindingFlags.Public);
    }

    public EntityMaterializer(DbTableInfo tableInfo) {
      _tableInfo = tableInfo;
    }
    public OutColumnMapping AddColumn(DbColumnInfo column, int readerColumnIndex) {
      var colMapping = new OutColumnMapping() { ReaderColumnIndex = readerColumnIndex, DbColumn = column };
      if (column.Flags.IsSet(DbColumnFlags.PrimaryKey))
        _primaryKeyColumns.Add(colMapping);
      else
        _nonKeyColumns.Add(colMapping);
      return colMapping; 
    }

    // Used for efficient reading entities in linq queries
    public object ReadEntity(IDataRecord record, EntitySession session) {
      // Some outer join queries may produce entities that are null; so we first try to read Primary key values - if they're all null, we return null. 
      var entRec = new EntityRecord(_tableInfo.Entity, EntityStatus.Loading);
      //First read primary key
      ReadColumns(record, entRec, _primaryKeyColumns);
      if (entRec.KeyIsNull(_tableInfo.Entity.PrimaryKey))
        return null;
      //Read the rest of the columns
      ReadColumns(record, entRec, _nonKeyColumns);
      var sessionRec = session.Attach(entRec); //might return different, previously loaded record
      return sessionRec.EntityInstance;
    }

    private void ReadColumns(IDataRecord record, EntityRecord intoRecord, IList<OutColumnMapping> columns) {
      //for-i loop is more efficient than foreach
      for (int i= 0; i < columns.Count; i++ ) {
        var colMap = columns[i];
        var v = record[colMap.ReaderColumnIndex];
        var convV = colMap.DbColumn.TypeInfo.ReadConverter(v);
        intoRecord.ValuesOriginal[colMap.DbColumn.Member.ValueIndex] = convV;
      }
    }

  }//class
}
