
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq.Expressions;

using Vita.Common;

namespace Vita.Data.Linq.Translation.Expressions
{
    /// <summary>
    /// A GroupExpression holds a grouped result
    /// It is usually transparent, except for return value, where it mutates the type to IGrouping
    /// </summary>
    [DebuggerDisplay("GroupExpression: {GroupExpression}")]
    public class GroupExpression : MutableExpression
    {
        public const ExpressionType ExpressionType = (ExpressionType)CustomExpressionType.Group;

        public Expression GroupedExpression { get; private set; }
        public Expression KeyExpression { get; private set; }

        public IList<ColumnExpression> Columns { get; private set; }

        public GroupExpression(Expression groupedExpression, Expression keyExpression, IList<ColumnExpression> columns)
            : base(ExpressionType, groupedExpression.Type)
        {
            GroupedExpression = groupedExpression;
            KeyExpression = keyExpression;
            Columns = columns; 
        }

        public bool IsDistinct {
          get { return GroupedExpression == KeyExpression; }
        }

        public override Expression Mutate(IList<Expression> newOperands)
        {
            if (newOperands.Count > 0)
                Util.Throw("S0065: Don't Mutate() a GroupExpression");
            return this;
        }
    }
}