﻿
using System;
using System.Linq;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;

using Vita.Data.Linq.Translation.Expressions;

namespace Vita.Data.Linq.Translation.Expressions
{
    /// <summary>
    /// A MetaTablePiece contains aliases for tables (used on joins)
    /// </summary>
    public class MetaTableExpression : MutableExpression
    {
      protected class Alias {
        public MemberInfo Member;
        public Expression Expression; 

      }
        public const ExpressionType ExpressionType = (ExpressionType)CustomExpressionType.MetaTable;

        // protected IDictionary<MemberInfo, MutableExpression> Aliases;
        //RI: changing to array to allow Mutate
        protected Alias[] Aliases; 

        public MutableExpression GetTableExpression(MemberInfo memberInfo)
        {
            var found = Aliases.FirstOrDefault(kv => kv.Member == memberInfo);
            if (found != null)
              return (MutableExpression) found.Expression;
            return null; 
        }

        public MetaTableExpression(IDictionary<MemberInfo, MutableExpression> aliases, Type metaTableType)
            : base(ExpressionType, metaTableType)
        {
            /* This check was disabled just becouse should be implied by the usage
            foreach (var alias in aliases)
            {
                if ( !(alias.Value is TableExpression) && !(alias.Value is MetaTableExpression))
                {
                    Util.Throw("Invalid MetaTable element type: {0}", alias.Value.Type);
                }
            }
             * */
            Aliases = aliases.Select(kv => new Alias() {Member = kv.Key, Expression = kv.Value}).ToArray();
        }

        public override IEnumerable<Expression> Operands {
          get {
            return Aliases.Select(a => a.Expression);
          }
        }

        //RI: adding this to handle conversion to final 'ObjectReader' delegate
        // NOTE: here we implicitly rely on match between order of Aliases and parameters of the constructor of anonymous type
        public NewExpression ConvertToNew(IList<Expression> newOperands) {
          var newValues = newOperands.ToArray();
          var members = Aliases.Select(a => a.Member).Reverse().ToArray();
          var constrList = this.Type.GetConstructors();
          var constr0 = constrList[0];
          var prms = constr0.GetParameters();
          var defaultConstr = constr0; // this.Type.GetConstructor(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance, null, Type.EmptyTypes, null); 
          var result = Expression.New(defaultConstr, newValues); //, members);
          return result; 
        }
    }
}