﻿
using System.Linq.Expressions;

using Vita.Common;
using Vita.Data.Linq.Translation.Expressions;

namespace Vita.Data.Linq.Translation {
    /// <summary>
    /// Analyzes Expressions before translation to SQL, to help the translator
    /// "Prequel" stands, of course, for "PreSQL"
    /// </summary>
    internal class PrequelAnalyzer 
    {
        /// <summary>
        /// Translates some generic CLR patterns to specific preSQL patterns
        /// </summary>
        /// <param name="expression"></param>
        /// <param name="builderContext"></param>
        /// <returns></returns>
        public virtual Expression Analyze(Expression expression, BuilderContext builderContext)
        {
            return expression.Recurse(e => AnalyzeExpression(e, builderContext));
        }

        protected virtual Expression AnalyzeExpression(Expression expression, BuilderContext builderContext)
        {
            var binExpr = expression as BinaryExpression;
            switch (expression.NodeType) {
              // string Add --> Concat
              case ExpressionType.Add:
                if (binExpr.Left.Type == typeof(string))
                  return new SpecialExpression(SpecialExpressionType.Concat, binExpr.Left, binExpr.Right);
                break;
            }//switch nodeType
            return expression;
        }//method 
    }
}