﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;
using Vita.Logging;
using Vita.Data.Model;

namespace Vita.Data.Management {

  public class DbSchemaUpdateInfo {
    public DbModel NewDbModel;
    public DbModel OldDbModel;
    public List<DbSchemaUpdateAction> Actions = new List<DbSchemaUpdateAction>();

    public DbSchemaUpdateInfo(DbModel newModel, DbModel oldModel) {
      NewDbModel = newModel;
      OldDbModel = oldModel;
    }
    public override string ToString() {
      return Util.SafeFormat("Model={0}, ChangeCount={1}", NewDbModel.EntityModel.ModelName, Actions.Count);
    }
    public string GetAllAsText() {
      var result = string.Join(Environment.NewLine, Actions);
      return result;
    }
  }

  //The order is important - this is the order in which changes are applied
  public enum DbSchemaChangeActionType {
    RoutineDrop,
    RefConstraintDrop,
    KeyDrop, //indexes, constraint
    TableDrop,
    SchemaDrop,
    SchemaAdd,
    ColumnDrop,
    TableAdd,
    TableRename,
    ColumnRename,
    ColumnInit,  // action to initialize NULL values when column changes to non-nullable
    ColumnModify,
    ColumnAdd,
    ColumnAddedInit, //special action for initializing new added columns
    ColumnAddComplete, //action to set new columns that are not nullable and have no default to NonNullable
    PrimaryKeyAdd,
    KeyAdd,
    RefConstraintAdd,
    RoutineAdd,
  }

  public class DbSchemaUpdateAction {
    public DbSchemaChangeActionType ActionType; 
    public object ObjectInfo;
    public DbSchemaUpdateAction(DbSchemaChangeActionType actionType, object objectInfo) {
      ActionType = actionType;
      ObjectInfo = objectInfo;
    }

    public override string ToString() {
      return ActionType + ", " + ObjectInfo;
    }
    //used in sorting
    public static int ByChangeType(DbSchemaUpdateAction x, DbSchemaUpdateAction y) {
      int xg = (int)x.ActionType;
      int yg = (int)y.ActionType;
      return xg.CompareTo(yg); 
    }
  }//class

}
