﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection.Emit;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.CSharp;

using Vita.Entities;
using Vita.Data;
using Vita.Data.Management;
using Vita.Data.Driver;
using Vita.Data.Model;
using Vita.Activation;
using Vita.Logging;
using Vita.Data.Model.Construction;
using Vita.Entities.Model.Construction;
using Vita.Entities.Runtime;

namespace Vita.SourceCodeGen {
  public static class SourceCodeGenUtil {

    public static CompilerResults CompileSources(Type driverType, params string[] sources) {
      var csProvider = new CSharpCodeProvider();
      var driverLoc = driverType.Assembly.Location;
      var driverAssembly = System.IO.Path.GetFileName(driverLoc); 
      var compilerParams = new CompilerParameters(new string[] { "System.dll", "System.Core.dll", "System.Data.dll", 
                                                                 "Vita.Core.dll", "Vita.Data.dll", driverAssembly });
      var results = csProvider.CompileAssemblyFromSource(compilerParams, sources);
      return results;
    }

    // Used for verification that generated c# entities produce identical database objects
    public static List<DbSchemaUpdateAction> CompareDatabaseSchemas(Assembly assembly, Type modelType, EntityCodeGeneratorConfig config,
                   DbDriver dbDriver, DbOptions dbOptions, string connectionString, Log log) {
      var entModel = Activator.CreateInstance(modelType) as EntityModel;
      EntityModelInfoBuilder.Build(entModel, log);
      var schemaOptions = DbSchemaUpdateOptions.VerifyKeysIndexes;
      var dbSettings = new DbSettings(dbDriver, dbOptions, connectionString, schemaOptions);
      var dbModel = DbModelBuilder.Build(entModel, dbSettings, log);
      var db = new Database(dbModel); 
      var updater = new DbSchemaUpdater(db, log); 
      var changes = updater.GetSchemaUpdates();
      return changes.Actions;
    }

    public static bool IsSet(this CodeGenOptions options, CodeGenOptions option) {
      return (options & option) != 0;
    }

  }//class

}
