﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data; 

using Vita.Entities.Model;
using Vita.Data.Model;
using Vita.Data.Driver; 

namespace Vita.Data.SqlCe {
  public class SqlCeSqlBuilder : DbSqlBuilder {
    public SqlCeSqlBuilder(DbModel dbModel) : base(dbModel) { }

    public override DbCommandInfo BuildSelectAllPagedCommand(EntityCommand entityCommand) {
      const string SqlSelectAllPaged = @"
SELECT {0}
  FROM {1}
  {2}
  OFFSET @__skiprows ROWS
  FETCH NEXT @__maxrows ROWS ONLY;
";

      var table = DbModel.LookupDbObject<DbTableInfo>(entityCommand.TargetEntityInfo, throwNotFound: true);
      //Build column list
      var cmdName = Settings.NamingPolicy.ConstructDbCommandName(entityCommand, table.TableName, "SelectAllPaged");
      var strColumns = table.Columns.GetSelectable().GetSqlNameList();
      string strOrderBy = (table.DefaultOrderBy == null) ?
        "ORDER BY " + table.PrimaryKey.Columns.GetSqlNameList() : BuildOrderBy(table.DefaultOrderBy);
      var sql = string.Format(SqlSelectAllPaged, strColumns, table.FullName, strOrderBy);
      var cmdInfo = CreateDbCommandInfo(entityCommand, cmdName, table, DbExecutionType.Query, sql);
      cmdInfo.Flags |= DbCommandFlags.WithPaging;
      return cmdInfo;
    }

  }//class

}
