﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;
using Vita.Entities;
using Vita.Entities.Model;
using Vita.Entities.Runtime;

namespace Vita.Modules.ChangeTracking {

  public class ChangeTrackingLog {

    public string Name;
    internal ChangeTrackingModule Module; 
    public HashSet<Type> TrackedEntities = new HashSet<Type>(); //tracked types
    public DataGrouper<Guid> Grouper; //might be null for non-grouped log
    private object _lockObject = new object();

    public ChangeTrackingLog(string name, IEnumerable<Type> trackedEntities, DataGrouper<Guid> partitioner = null) {
      Name = name;
      TrackedEntities.UnionWith(trackedEntities); 
      Grouper = partitioner;
    }
    public override string ToString() {
      return Name;
    }

    internal void Init(ChangeTrackingModule module) {
      Module = module; 
    }
    public bool IsGrouped() {
      return Grouper != null; 
    }

    internal IChangeTrackEntry CreateLogEntry(IEntitySession session, Guid transactionId, Guid partitionId,  string changes, ChangleLogEntryType entryType) {
      var entry = session.NewEntity<IChangeTrackEntry>();
      entry.LogName = this.Name;
      entry.TransactionId = transactionId;
      entry.Partition = partitionId;
      entry.EntryType = entryType; 
      entry.Changes = changes;
      return entry;
    }

    internal void OnEntitySessionSaving(EntitySession session) {
      var trackedRecords = FilterTrackedEntities(session.RecordsChanged);
      if (trackedRecords.Count == 0)
        return;
      var tranId = Guid.NewGuid(); 
      var groupedRecords = GroupRecords(trackedRecords);
      //Add change log records
      var serilizer = Module.ChangeLogSerializer; 
      foreach (var de in groupedRecords) {
        var records = de.Value;
        if (records.Count == 0)
          continue; 
        //build changeInfo list
        var changeInfoList = new List<EntityChangeInfo>();
        foreach (var rec in records)
          changeInfoList.Add(new EntityChangeInfo(rec));
        var changes = serilizer.Serialize(changeInfoList);
        var changeEntity = CreateLogEntry(session, tranId, de.Key, changes, ChangleLogEntryType.Changes); 
      }
    }//method

    private List<EntityRecord> FilterTrackedEntities(IList<EntityRecord> records) {
      var result = new List<EntityRecord>();
      foreach (var rec in records) {
        var entType = rec.EntityInfo.EntityType;
        if (TrackedEntities.Contains(entType))
          result.Add(rec); 
      }
      return result; 
    }

    private Dictionary<Guid, IList<EntityRecord>> GroupRecords(IList<EntityRecord> records) {
      if (Grouper == null) {
        var result = new Dictionary<Guid, IList<EntityRecord>>();
        result.Add(Guid.Empty, records);
        return result;
      } else
        return Grouper.GroupRecords(records); 
    }

  }//class


}
