﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities.Runtime;

namespace Vita.Modules.ChangeTracking {

  public enum EntityChangeType {
    New,
    Update,
    Delete,
  }

  /// <summary>Identifies a changed entity and change type. </summary>
  public class EntityChangeInfo {
    public EntityChangeType ChangeType;
    public string EntityName;
    public string KeyValues; 

    public EntityChangeInfo() { }

    public EntityChangeInfo(EntityRecord record) {
      ChangeType = ToChangeType(record.Status);
      EntityName = record.EntityInfo.FullName;
      KeyValues = record.PrimaryKey.ValuesToString();
    }
    public EntityChangeInfo(EntityKey primaryKey, EntityChangeType changeType) {
      EntityName = primaryKey.KeyInfo.Entity.FullName;
      KeyValues = primaryKey.ValuesToString();
      ChangeType = changeType;
    }

    public override string ToString() {
      return ChangeType + "/" + EntityName + "/" + KeyValues;
    }
    public string GetFullKey() {
      return EntityName + "/" + KeyValues;
    }

    private EntityChangeType ToChangeType(EntityStatus status) {
      switch (status) {
        case EntityStatus.New: return EntityChangeType.New;
        case EntityStatus.Deleting: return EntityChangeType.Delete;
        default: return EntityChangeType.Update;
      }
    }
  } //class

  public class EntityChangeSet {
    public Guid Partition;
    public DateTime FromDateTime;
    public DateTime TillDateTime; 
    public List<EntityChangeInfo> Changes = new List<EntityChangeInfo>();
  }
  
}
