﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Common;
using Vita.Entities.Model;
using Vita.Entities.Runtime;

namespace Vita.Modules.DataSync {

  // Holds information about properties that must be cleared in exported or imported entities
  public class ClearValueRule {
    public readonly Type EntityType;
    public readonly string[] PropertyNames;
    internal List<EntityMemberInfo> Members = new List<EntityMemberInfo>();

    public ClearValueRule(Type entityType, string[] propertyNames) {
      EntityType = entityType;
      PropertyNames = propertyNames;
    }

    public void Init(string channelName, EntityStore entityStore) {
      var ent = entityStore.Model.ModelInfo.GetEntityInfo(EntityType);
      Util.Check(ent != null, "Type {0} specified in ClearValueRule in sync channel {1} is not registered as an entity.", EntityType, channelName);
      // Convert property names into memberInfo objects
      foreach (var propName in PropertyNames) {
        var member = ent.GetMember(propName);
        Util.Check(member != null, "Property {0} referenced in ClearValueRule in sync channel {1} is not found on entity {2}.", 
            propName, channelName, ent.EntityType.Name);
        Members.Add(member);
      }
    }//method

  }//class
}
