﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;

namespace Vita.Modules.DataSync {

  public enum SyncType {
    Import,
    Export,
  }

  public enum SyncStatus {
    InProgress, // sync pack was sent to the client, waiting for confirmation
    Success,
    Error,
  }

  [Entity, ClusteredIndex("CreatedOn,Id"), OrderBy("CreatedOn:DESC"), IsSystem]
  public interface IDataSyncLog {
    [PrimaryKey]
    Guid Id { get; set; }

    [Auto(AutoType.CreatedOn), Utc]
    DateTime CreatedOn { get; set; }
    
    string ChannelName { get; set; }

    SyncType SyncType { get; set; }
    SyncStatus Status { get; set; }

    [Utc]
    DateTime ChangesSince { get; set; }
    [Utc]
    DateTime ChangesUntil { get; set; }
    
    Guid PartitionId { get; set; }
    string RemoteParty { get; set; }
    int RecordCount { get; set; }
    
    [Memo, Nullable]
    string Messages { get; set; } //diagnostic messages
  }

  public static class DataSyncExtensions {
    public static IDataSyncLog NewDataSyncLog(this IEntitySession session, string channelName, Guid syncSetId, 
           SyncType syncType, SyncStatus status, string remoteParty, Guid partitionId, DateTime since, DateTime until, int recordCount){
      var syncLog = session.NewEntity<IDataSyncLog>();
      syncLog.Id = syncSetId;
      syncLog.ChannelName = channelName;
      syncLog.SyncType = syncType;
      syncLog.Status = status;
      syncLog.RemoteParty = remoteParty;
      syncLog.PartitionId = partitionId;
      syncLog.ChangesSince = since;
      syncLog.ChangesUntil = until;
      syncLog.RecordCount = recordCount;
      return syncLog;    
    }
    public static bool IsSet(this ImportSyncActions actions, ImportSyncActions action) {
      return (actions & action) != 0;
    }
  }

}
