﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Xml;
using Vita.Data;
using Vita.Entities;
using Vita.Entities.Runtime; 

namespace Vita.Modules.ErrorLogging {
  // Constructs HTML pages for error list and error details. 
  // Contains HTML-producing methods that are hooked as handlers in web service for requests for error pages. 
  public class ErrorLogViewer {
    public readonly IEntityStore LogStore;
    public string ErrorListPageUrlTemplage;
    public string ErrorDetailsPageUrlTemplate;

    public ErrorLogViewer(IEntityStore logStore,
                          string errorListPageUrlTemplate = "/errors?skip={0}&take={1}",
                          string errorDetailsPageUrlTemplate = "/error/{0}") {
      LogStore = logStore;
      ErrorListPageUrlTemplage = errorListPageUrlTemplate;
      ErrorDetailsPageUrlTemplate = errorDetailsPageUrlTemplate;
    }

    public string GetErrorListPage(Uri requestUri, NameValueCollection parameters) {
      var paging = GetPagingParameters(parameters, 50);
      var session = LogStore.OpenSession();
      var errors = session.GetEntities<IErrorInfo>(paging.Skip, paging.Take).ToList();
      var pageBuilder = new ErrorHtmlPageBuilder(ErrorListPageUrlTemplage, ErrorDetailsPageUrlTemplate);
      pageBuilder.CreateErrorListPage(errors, paging.Skip, paging.Take);
      return pageBuilder.GetHtml();
    }

    public string GetErrorDetailsPage(Uri requestUri, NameValueCollection parameters) {
      var session = LogStore.OpenSession();
      var sUri = requestUri.ToString();
      var segms = sUri.Split('/');
      var lastSegm = segms[segms.Length - 1];
      Guid errId;
      Guid.TryParse(lastSegm, out errId);
      var error = session.GetEntity<IErrorInfo>(errId);
      var pageBuilder = new ErrorHtmlPageBuilder(ErrorListPageUrlTemplage, ErrorDetailsPageUrlTemplate);
      pageBuilder.CreateErrorDetailsPage(error);
      return pageBuilder.GetHtml();
    }

    public static Paging GetPagingParameters(NameValueCollection parameters, int maxAllowed = 1000) {
      return ParsePaging(parameters["skip"], parameters["take"], maxAllowed);
    }

    public static Paging ParsePaging(string skip, string take, int maxAllowed = 1000) {
      int iSkip, iMaxCount;
      if (!int.TryParse(skip, out iSkip))
        iSkip = 0;
      if (!int.TryParse(take, out iMaxCount))
        iMaxCount = int.MaxValue;
      if (iMaxCount > maxAllowed)
        iMaxCount = maxAllowed;
      return new Paging(iSkip, iMaxCount);
    }

  }//class
}
