﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vita.Entities;

namespace Vita.Modules.Login {
  using Binary = System.Data.Linq.Binary;

  [Flags]
  public enum LoginFlags {
    None = 0x00,
    AutoGenerated = 1 << 1,
    ForceChange = 1 << 2,
    Temporary = 1 << 3,

    EmailConfirmed = 1 << 4,
  }

  //  PasswordHash column is marked with Secret attribute - the value cannot be read, it is excluded from all Select statements and from LINQ mappings.
  //  This is a protection against leaking from the database. 

  [Entity, ClusteredIndex("CreatedOn,Id"), Unique("UserName,DomainId"), IsSystem]
  [Index("DomainId,UserNameHash,UserName")]
  public interface ILogin {
    [PrimaryKey, Auto]
    Guid Id { get; set; } 
    [Auto(AutoType.CreatedOn), Utc]
    DateTime CreatedOn { get; set; }

    //Plain-text userName; might be null if user names are confidential (ex: emails are used as user names)
    [Nullable]
    string UserName { get; set; }

    int UserNameHash { get; set; }

    [Size(LoginModule.HashSize), Nullable, Secret]
    Binary PasswordHash { get; set; }
    Guid DomainId { get; set; }

    DateTime? Expires { get; set; }
    LoginFlags Flags { get; set; }
  }

}
