﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;
using Vita.Modules.Login;

namespace Vita.Modules.PasswordReset {
  // Work in progress, to be completed

  // An excellent guide to managing logins and passwords:
  // http://www.troyhunt.com/2012/05/everything-you-ever-wanted-to-know.html

  [Entity, IsSystem]
  public interface ISecretQuestion {
    [PrimaryKey, Auto]
    Guid Id { get; set; }
    [Size(250)]
    string Question { get; set; }
    bool HasHint {get; set;}
  }

  [Entity, ClusteredIndex("Login,Id"), IsSystem]
  public interface ISecretQuestionAnswer {
    [PrimaryKey, Auto]
    Guid Id { get; set; }
    ILogin Login { get; set; }
    [Unique]
    ISecretQuestion Question { get; set; }
    [Size(30)]
    string Hint { get; set; } //Additional hint. For example for question "Your family member mobile phone?", hint would be 'wife'.
    [Size(LoginModule.HashSize), Secret]
    byte[] AnswerHash { get; set; }
  }

  public enum PasswordResetStatus {
    Initiated,
    InstructionsSent, 
    Responded, // clicked on URL in instructions
    Completed, //successfully completed, sent confirmation email
    Expired,
  }

  [Entity, ClusteredIndex("CreatedOn,Id"), IsSystem]
  public interface IPasswordResetJournal {
    [PrimaryKey, Auto]
    Guid Id { get; set; }
    [Auto(AutoType.CreatedOn), Utc]
    DateTime CreatedOn { get; set; }
    [Utc]
    DateTime Expires { get; set; }
    long IPAddress { get; set; }
    PasswordResetStatus Status { get; set; }
    [Size(50)]
    string Email { get; set; } //initial email entered by user for recovery
    [Size(50)]
    string RecoveryToken { get; set; } //to include in Instructions URL; or for SMS - secret code sent by sms, to enter on the page
  }


}
