﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Vita.Entities;

namespace Vita.Samples.BooksModel {

  public static class SampleDataGenerator {

    public static void CreateSampleData(IEntityStore entityStore) {
      var session = entityStore.OpenSession();

      var msPub = session.NewPublisher("MS Books"); //we are using extension method here
      var kidPub = session.NewPublisher("Kids Books");
      var johnBio = ConstructLongText(4000);
      var authorJohn = session.NewAuthor("John", "Sharp", johnBio);
      var authorJack = session.NewAuthor("Jack", "Pound");
      var authorJim = session.NewAuthor("Jim", "Hacker"); //this author is not user - we'll use this author to check some tricky query in tests

      var pubDate = DateTime.Today.AddYears(-1);
      //Books on programming from MS Books
      var csBook = session.NewBook(BookEdition.Paperback | BookEdition.EBook, BookCategory.Programming,
                 "c# Programming", "Expert programming in c#", msPub, pubDate, 20.0m);
      // Some multiline text in Abstract
      csBook.Abstract = @"Expert guide to programming in c# 4.0. 
Highly recommended for beginners and experts.
Covers c# 4.0.";
      csBook.Authors.Add(authorJohn); //this is many-to-many
      csBook.Authors.Add(authorJack);
      var vbBook = session.NewBook(BookEdition.Paperback | BookEdition.Hardcover, BookCategory.Programming,
                          "VB Programming", "Expert programming in VB", msPub, pubDate, 25.0m);
      vbBook.Authors.Add(authorJack);
      //Folk tale, no authors
      var kidBook = session.NewBook(BookEdition.Hardcover, BookCategory.Kids,
           "Three little pigs", "Folk tale", kidPub, pubDate, 10.0m);
      var winBook = session.NewBook(BookEdition.Hardcover, BookCategory.Programming,
           "Windows programming", "Introduction to Windows Programming", msPub, pubDate.AddYears(-10), 30.0m);
      winBook.Authors.Add(authorJohn);
      //Coupons
      var coupon1 = session.NewCoupon("C1", 10, DateTime.Now.AddMonths(1));
      var coupon2 = session.NewCoupon("C2", 10, DateTime.Now.AddMonths(1));
      var coupon3 = session.NewCoupon("C3", 10, DateTime.Now.AddMonths(1));
      //Create users
      var dora = session.NewUser("Dora", UserType.WebVisitor);
      var diego = session.NewUser("Diego", UserType.WebVisitor, false);
      var linda = session.NewUser("linda", UserType.BookEditor);
      var charlie = session.NewUser("charlie", UserType.StoreAdmin);
      //create users for authors
      authorJohn.User = session.NewUser("john", UserType.Author);
      authorJack.User = session.NewUser("jack", UserType.Author);
      try {
        session.SaveChanges();  //Save books, users and coupons
      } catch (ValidationException ex) {
        var msgs = ex.GetMessages();
        Debug.WriteLine(msgs);
        throw;
      }

      //Orders
      var doraOrder = session.NewOrder(dora);
      doraOrder.Add(csBook, 1);
      doraOrder.Add(kidBook, 2);
      doraOrder.CompleteOrder("C1");

      var diegoOrder = session.NewOrder(diego);
      diegoOrder.Add(vbBook, 1);
      diegoOrder.Add(csBook, 1);
      diegoOrder.CompleteOrder();
      //Reviews
      var doraReview = session.NewReview(dora, csBook, 5, "Very interesting book!", "Liked it very much!");
      var diegoReview = session.NewReview(diego, vbBook, 1, "Not worth it.", "Did not like it at all.");

      try {
        session.SaveChanges();  //Save books and authors
      } catch (ValidationException ex) {
        var msgs = ex.GetMessages(); 
        Debug.WriteLine(msgs);
        throw;
      }
    }

    private static string ConstructLongText(int length) {
      const string loremIpsum = @"
Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. 
Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. 
Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. 
Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.";
      int numCopies = length / loremIpsum.Length + 1;
      var sb = new StringBuilder(length);
      for (int i = 0; i < numCopies; i++)
        sb.Append(loremIpsum);
      return sb.ToString().Substring(0, length);
    }

  }
}
