﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;

namespace Vita.Tutorial.Part1 {
  // Demo-ed features:
  //   Defining simple entities (Book and Publisher); basic CRUD operations, loading entities by ID, using LINQ-2-SQL for
  //   selecting entities from database; lasy loading of entity references. 

  //Define entitites: [Entity] and [PrimaryKey] are the only attributes required for definition of a minimal entity model.
  [Entity]
  public interface IBook {
    [PrimaryKey]
    Guid Id { get; set; }
    string Author { get; set; }
    string Title { get; set; } //uses default length 50
    DateTime PublishedOn {get; set;}
    double Price { get; set; }
    IPublisher Publisher { get; set; }
  }

  [Entity]
  public interface IPublisher {
    [PrimaryKey]
    Guid Id { get; set; }
    string Name { get; set; }
  }

  public class MainBooksModule : EntityModule {
    public MainBooksModule(EntityArea area) : base(area, "Main") {
      RegisterEntities(typeof(IBook), typeof(IPublisher));
    }
  }

  public class Part1EntityModel : EntityModel {
    public MainBooksModule Main; 
    public Part1EntityModel() {
      var mainArea = AddArea("Part1", "part1");
      Main = new MainBooksModule(mainArea);
    }
  }

}//ns
