﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;

namespace Vita.Tutorial.Part2 {

  [Flags]
  public enum BookEdition {
    Paperback = 0x01,
    Hardcover = 0x02,
    EBook = 0x04,
  }

  public enum BookCategory {
    Programming,
    Fiction,
    Kids,
  }

  [Entity, OrderBy("Name")]
  public interface IPublisher {
    [PrimaryKey, Auto]
    Guid Id { get; } // no need for setter with Auto attribute
    string Name { get; set; }
    IList<IBook> Books { get; }
  }

  [Entity, OrderBy("PublishedOn:DESC"), Paged]
  public interface IBook {
    [PrimaryKey, Auto]
    Guid Id { get; } 
    [Size(50)]
    string Title { get; set; } 
    DateTime PublishedOn { get; set; }
    [Size(250), Nullable]
    string Description { get; set; }
    [Memo, Nullable]
    string Abstract { get; set; }
    BookCategory Category { get; set; }
    BookEdition Editions { get; set; }
    double Price { get; set; }
    IPublisher Publisher { get; set; }

    [ManyToMany(typeof(IBookAuthor))]
    IList<IAuthor> Authors { get; }
  } 

  [Entity, Paged, OrderBy("LastName,FirstName")]
  public interface IAuthor {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string FirstName { get; set; }
    string LastName { get; set; }
    [ManyToMany(typeof(IBookAuthor))]
    IList<IBook> Books { get; }
    [Computed(typeof(BookExtensions), "GetFullName"), DependsOn("FirstName,LastName")]
    string FullName { get; }
  }

  [Entity, PrimaryKey("Book,Author")]
  public interface IBookAuthor {
    IBook Book { get; set; }
    IAuthor Author { get; set; }
  }

  public class MainBooksModule : EntityModule {
    public MainBooksModule(EntityArea area)  : base(area, "Main") {
      RegisterEntities(typeof(IBook), typeof(IPublisher), typeof(IAuthor), typeof(IBookAuthor));
    }
  }

  public class Part2EntityModel : EntityModel {
    public MainBooksModule Main;
    public Part2EntityModel()  {
      var mainArea = AddArea("Part2", "part2");
      Main = new MainBooksModule(mainArea);
    }
  }



}//ns
