﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

using Vita.Entities;

namespace Vita.Tutorial.Part3 {

  [Flags]
  public enum BookEdition {
    Paperback = 0x01,
    Hardcover = 0x02,
    EBook = 0x04,
  }

  public enum BookCategory {
    Programming,
    Fiction,
    Kids,
  }

  [Entity, OrderBy("Name")]
  public interface IPublisher {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string Name { get; set; } //Unique key on Name is specified in IPublisherKeys companion type
    IList<IBook> Books { get; }
  }

  [Entity, Paged, OrderBy("PublishedOn:DESC")]
  public interface IBook {
    
    [PrimaryKey, Auto]
    Guid Id { get; }
    
    [Size(50)] 
    string Title { get; set; }
 
    [Column(DbType = DbType.Date)]
    DateTime PublishedOn { get; set; }

    [Size(250), Nullable]
    string Description { get; set; }
    
    [Memo, Nullable]
    string Abstract { get; set; }
    
    BookCategory Category { get; set; }
    
    BookEdition Editions { get; set; }
    
    double Price { get; set; }
    
    IPublisher Publisher { get; set; }

    [ManyToMany(typeof(IBookAuthor))]
    IList<IAuthor> Authors { get; }
  } 

  [Entity, Paged, OrderBy("LastName,FirstName")]
  public interface IAuthor {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string FirstName { get; set; }
    string LastName { get; set; }
    [ManyToMany(typeof(IBookAuthor))]
    IList<IBook> Books { get; }
  }

  [Entity, PrimaryKey("Book,Author")]
  public interface IBookAuthor {
    IBook Book { get; set; }
    IAuthor Author { get; set; }
  }

  public class MainBooksModule : EntityModule {
    public MainBooksModule(EntityArea area) : base(area, "Main") {
      RegisterEntities(typeof(IBook), typeof(IPublisher), typeof(IAuthor), typeof(IBookAuthor));
      //Register companion types that describe keys and indexes on entities
      this.RegisterCompanionTypes(typeof(IBookKeys), typeof(IAuthorKeys), typeof(IPublisherKeys));
    }
  }

  public class Part3EntityModel : EntityModel {
    public MainBooksModule Main;
    public Part3EntityModel() {
      var mainArea = AddArea("Part3", "part3");
      Main = new MainBooksModule(mainArea);
    }
  }


}//ns
