﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;

using Vita.Activation;
using Vita.Data;

namespace Vita.Tutorial.Part3 {
  //Demo-ed features:
  // Using attributes to create database indexes (unique, clustered)

  public static class Part3_Sample {

    public static void RunSample() {
      Console.Clear();
      Console.WriteLine("Running Sample.");
      Setup.SchemaManager.DropSchemaObjects("part3");

      Console.WriteLine("Opening an entity store...");
      var model = new Part3EntityModel();
      var entityStore = model.CreateEntityStore(new Vita.Data.MsSql.MsSqlDriver(), Setup.ConnectionString);
      Console.WriteLine("Success. Updated database schema, opened entity store.");
      //Print all indexes created from attributes
      Console.WriteLine("Printing created keys and indexes...");
      var indexes = GetIndexes("part3");
      foreach (DataRow row in indexes.Rows) 
        Console.WriteLine("  " + GetIndexInfo(row));
      Console.WriteLine("Demo completed. ");
      Console.WriteLine();
    }

    private static string GetIndexInfo(DataRow row) {
      var name = (string)row["index_name"];
      var result = (name + ":").PadRight(25);
      if ((bool)row["primary_key"]) result += " PrimaryKey";
      if ((bool)row["unique"]) result += " Unique";
      if((string)row["clustered"] == "CLUSTERED") result += " Clustered";
      return result;  

    }

    public static DataTable GetIndexes(string schema) {
      const string getIndexesTemplate = @"
SELECT ss.name AS table_schema, st.name AS table_name, si.name AS index_name, 
       si.is_primary_key as [primary_key], si.type_desc as [clustered], 
       si.is_unique as [unique], si.is_disabled as [disabled]
FROM   sys.indexes AS si INNER JOIN
       sys.tables AS st ON si.object_id = st.object_id INNER JOIN
       sys.schemas AS ss ON ss.schema_id = st.schema_id
WHERE (si.name IS NOT NULL) AND (ss.name = '{0}')
ORDER BY table_schema, table_name, index_name
";
      var sql = string.Format(getIndexesTemplate, schema);
      var conn = new SqlConnection(Setup.ConnectionString);
      var cmd = conn.CreateCommand();
      cmd.CommandText = sql;
      cmd.CommandType = CommandType.Text;
      conn.Open();
      var reader = cmd.ExecuteReader();
      var tbl = new DataTable("Indexes");
      tbl.Load(reader);
      conn.Close();
      return tbl;
    }



  }//class
}//ns
