﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;

namespace Vita.Tutorial.Part4 {
  //Helper methods to create entities
  public static class BookExtensions {
    public static IBook NewBook(this IEntitySession session, BookEdition editions, BookCategory category, string title, string description,
         IPublisher publisher, DateTime publishedOn, double price) {
      var book = session.NewEntity<IBook>();
      book.Editions = editions;
      book.Category = category; 
      book.Title = title;
      book.Description = description;
      book.Publisher = publisher;
      book.PublishedOn = publishedOn;
      book.Price = price;
      return book;
    }

    public static IPublisher NewPublisher(this IEntitySession session, string name) {
      var pub = session.NewEntity<IPublisher>();
      pub.Name = name;
      return pub;
    }

    public static IAuthor NewAuthor(this IEntitySession session, string firstName, string lastName) {
      var auth = session.NewEntity<IAuthor>();
      auth.FirstName = firstName;
      auth.LastName = lastName;
      return auth;
    }

    // Static method computing FullName computed property for an Author
    public static string GetFullName(IAuthor author) {
      return author.FirstName + " " + author.LastName;
    }

    // Static method validating Book entity
    public static void ValidateBook(EntityValidator validator, IBook book) {
      validator.Check(book.Price > 0.01, book.Price, "Book price must be greater than 1 cent.", 
          null, "PriceNegative", "Price", book);
    }
  }//class
}
