﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using Vita.Entities;

namespace Vita.Tutorial.Part4 {

  [Flags]
  public enum BookEdition {
    Paperback = 0x01,
    Hardcover = 0x02,
    EBook = 0x04,
  }

  public enum BookCategory {
    Programming,
    Fiction,
    Kids,
  }

  [Entity, OrderBy("Name")]
  public interface IPublisher {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string Name { get; set; }
    IList<IBook> Books { get; }
  }

  [Entity, Paged, OrderBy("PublishedOn:DESC")]
  [Validate(typeof(BookExtensions), "ValidateBook")]  
  public interface IBook {
    [PrimaryKey, Auto]
    Guid Id { get; }
    [Size(50), Index] //index titles for fast search
    string Title { get; set; } 
    
    [ClusteredIndex, Column(DbType = DbType.Date)]
    DateTime PublishedOn { get; set; }

    [Size(250), Nullable]
    string Description { get; set; }
    
    [Memo, Nullable]
    string Abstract { get; set; }
    
    BookCategory Category { get; set; }
    
    BookEdition Editions { get; set; }
    
    double Price { get; set; }
    
    IPublisher Publisher { get; set; }

    [ManyToMany(typeof(IBookAuthor))]
    IList<IAuthor> Authors { get; }
  } 

  [Entity, Paged, OrderBy("LastName,FirstName"), ClusteredIndex("LastName,Id")]
  public interface IAuthor {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string FirstName { get; set; }
    [Index] //for quick search by last name
    string LastName { get; set; }
    [ManyToMany(typeof(IBookAuthor))]
    IList<IBook> Books { get; }
    [Computed(typeof(BookExtensions), "GetFullName")] 
    string FullName { get; }
  }

  [Entity, PrimaryKey("Book,Author")]
  public interface IBookAuthor {
    IBook Book { get; set; }
    IAuthor Author { get; set; }
  }


  public class MainBooksModule : EntityModule {
    public MainBooksModule(EntityArea area): base(area, "Main") {
      RegisterEntities(typeof(IBook), typeof(IPublisher), typeof(IAuthor), typeof(IBookAuthor));
    }
  }

  public class Part4EntityModel : EntityModel {
    public MainBooksModule Main;
    public Part4EntityModel() {
      var mainArea = AddArea("Part4", "part4");
      Main = new MainBooksModule(mainArea);
    }
  }

}//ns
