﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text; 

using Vita.Entities;

namespace Vita.Tutorial.Part5 {

  public class BooksModule : EntityModule {
    IEntityCommand _getBooksByAuthorCommand, _changePriceCommand;

    public BooksModule(EntityArea area) : base(area, "Books", "Books module") {
      //Register entity types
      this.RegisterEntities(typeof(IBook), typeof(IPublisher), typeof(IAuthor), typeof(IBookAuthor)); 
      
      //Register companion types that describe keys and indexes on entities
      this.RegisterCompanionTypes(
          typeof(IBookKeys), typeof(IAuthorKeys), typeof(IPublisherKeys));

      //Create custom commands - they will be compiled into stored procedures
      var books = this.EntitySet<IBook>();
      var authors = this.EntitySet<IAuthor>();
      var bookAuthors = this.EntitySet<IBookAuthor>();
      var qryBooksByAuthor = from bk in books
                             join ba in bookAuthors on bk equals ba.Book
                             join au in authors on ba.Author equals au
                             where au.LastName == Param<string>(30)
                             select bk;
      _getBooksByAuthorCommand = this.CreateSelect<IBook>("BooksGetByAuthorLastName", 
              "Gets books for author(s) with a given last name.", qryBooksByAuthor);
      var qryChangePrice = from bk in books
                             where bk.Category == Param<BookCategory>()
                             select new { Id = bk.Id, Price = bk.Price * Param<double>() };
      _changePriceCommand = this.CreateUpdate<IBook>("BooksChangePrice", 
              "Changes prices of books in a given category.", qryChangePrice);
    }

    //Methods executing stored procedures
    public IList<IBook> GetBooksByAuthor(IEntitySession session, string lastName) {
      return session.ExecuteQuery<IBook>(_getBooksByAuthorCommand, lastName);
    }

    public void ChangePrice(IEntitySession session, BookCategory category, double changePerc) {
      var factor = (100.0 + changePerc) / 100.0d;
      session.ExecuteNonQuery(_changePriceCommand, category, factor);
    }

  }//BooksModule 

}
