﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using Vita.Entities;

namespace Vita.Tutorial.Part5 {

  [Flags]
  public enum BookEdition {
    Paperback = 0x01,
    Hardcover = 0x02,
    EBook = 0x04,
  }

  public enum BookCategory {
    Programming,
    Fiction,
    Kids,
  }

  [Entity, OrderBy("Name")]
  public interface IPublisher {
    [PrimaryKey, Auto]
    Guid Id { get; }
    [Unique(IndexName = "PubName")]
    string Name { get; set; }
    IList<IBook> Books { get; }
  }

  [Entity, Paged, OrderBy("PublishedOn:DESC")]
  [Validate(typeof(BookExtensions), "ValidateBook")]
  public interface IBook {
    [PrimaryKey, Auto]
    Guid Id { get; }

    [Size(50)] 
    string Title { get; set; }

    [Column(DbType = DbType.Date)]
    DateTime PublishedOn { get; set; }
    
    [Size(250), Nullable]
    string Description { get; set; }
    
    [Memo, Nullable]
    string Abstract { get; set; }
    
    BookCategory Category { get; set; }
    
    BookEdition Editions { get; set; }
    
    double Price { get; set; }
    
    IPublisher Publisher { get; set; }

    [ManyToMany(typeof(IBookAuthor))]
    IList<IAuthor> Authors { get; }
  } 

  [Entity, Paged, OrderBy("LastName,FirstName"), ClusteredIndex("LastName,Id")]
  public interface IAuthor {
    [PrimaryKey, Auto]
    Guid Id { get; }
    string FirstName { get; set; }
    string LastName { get; set; }
    [ManyToMany(typeof(IBookAuthor))]
    IList<IBook> Books { get; }
    [Computed(typeof(BookExtensions), "GetFullName")] 
    string FullName { get; }
  }

  [Entity, PrimaryKey("Book,Author")]
  public interface IBookAuthor {
    IBook Book { get; set; }
    IAuthor Author { get; set; }
  }

}//ns
