﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Vita.Entities;
using Vita.Data;
using Vita.Modules.ErrorLogging;

namespace Vita.Tutorial.Part5 {
  // Demo-ed features:
  // Entity modules, custom methods

  // NOTE: temporarily disabled; custom stored procedure feature is disabled in the framework, being refactored.
  public static class Part5_Sample {

    public static void RunSample() {
      Console.Clear();
      Console.WriteLine("Running Sample.");
      Setup.SchemaManager.DropSchemaObjects("part5");
      Setup.SchemaManager.DropSchemaObjects("log");

      Console.WriteLine("Creating books module, opening entity store...");

      var entityModel = new BooksEntityModel();
      var entityStore = entityModel.CreateEntityStore(Setup.ConnectionString, Setup.DbDriver);
      Console.WriteLine("Success. Updated database schema, opened entity store.");
      
      Console.WriteLine("Creating sample entities: publisher, authors, books.");
      var session = entityStore.OpenSession();
      //publishers
      var msPub = session.NewPublisher("MS Publishing");
      //authors
      var john = session.NewAuthor("John", "Sharp");
      var jack = session.NewAuthor("Jack", "Pound");
      //Books
      var csBook = session.NewBook(BookEdition.Hardcover, BookCategory.Programming, "c# Programming", "Expert c# programming", msPub, DateTime.Today.AddYears(-1), 25); 
      csBook.Authors.Add(john);
      var vbBook = session.NewBook(BookEdition.EBook | BookEdition.Paperback, BookCategory.Programming, "VB Programming", "Expert VB programming", msPub, 
         DateTime.Today, 20); 
      vbBook.Authors.Add(jack);
      vbBook.Authors.Add(john);
      //remember vbBook ID
      var vbBookId = vbBook.Id; 

      session.SaveChanges(); //Submit to database
      Console.WriteLine("Success. Created a publisher and 2 books.");
      Console.WriteLine();

      Console.WriteLine("Using custom methods/stored procedures - retrieving all books by John Sharp ");
      session = entityStore.OpenSession(); //open new session, to make sure we read fresh entities from database
      var booksByJohnSharp = entityModel.MainModule.GetBooksByAuthor(session, "Sharp");
      foreach (var book in booksByJohnSharp)
        Console.WriteLine("  Book: " + book.Title);
      Console.WriteLine();

      Console.WriteLine("Using custom methods/stored procedures - changing price for multiple books.");
      Console.WriteLine("  Reducing price of Programming books by 20%.");
      entityModel.MainModule.ChangePrice(session, BookCategory.Programming, -20);
      Console.Write("  Verifying VB book price... ");
      session = entityStore.OpenSession();
      vbBook = session.GetEntity<IBook>(vbBookId);
      //The price should change from 20 to 16
      if (Math.Abs(vbBook.Price - 16) > 0.01)  throw new Exception("Error: book price did not change.");
      Console.WriteLine("  Success - VB book price changed from 20 to 16.");
      Console.WriteLine();

      Console.WriteLine("Demo completed. ");
      Console.WriteLine();
    }



  }//class
}//ns
