﻿using System;
using System.Diagnostics;
using System.Data.SqlClient;
using System.Threading;
using System.CodeDom.Compiler;
using System.Linq;
using System.Reflection;
using System.Collections.Generic;
using Microsoft.CSharp;

using Vita.Entities;
using Vita.Data;

namespace Vita.Samples.BooksSample.UnitTests {
#if USE_NUNIT
  using NUnit.Framework;
  using TestClass = NUnit.Framework.TestFixtureAttribute;
  using TestMethod = NUnit.Framework.TestAttribute;
  using TestInitialize = NUnit.Framework.SetUpAttribute;
  using TestCategory = NUnit.Framework.CategoryAttribute;
#else
  using Microsoft.VisualStudio.TestTools.UnitTesting;
  using System.IO;
  using Vita.SourceCodeGen;
#endif


#if MSSQL
  [TestClass]
#endif
  public class DbToEntitiesSourceCodeGenTests {

    [TestInitialize]
    public void TestInit() {
      SetupHelper.Init();
    }

    [TestMethod]
    public void TestDbToEntitiesSourceGenerator() {
      var cfg = new EntityCodeGeneratorConfig(SetupHelper.GetDriver(), SetupHelper.GetConnectionString(), "Vita.Samples.BooksModelGenerated", "BooksModel",  @"BooksModelGenerated.cs",
                                              CodeGenOptions.Default | CodeGenOptions.UtcDates, 50, new string[] {"books", "log"});
      var entModelGen = new EntityModelReverseGenerator();
      var entModel = entModelGen.Generate(cfg);
      Assert.IsNotNull(entModel, "Failed to generate entity model.");
      var codeGen = new EntityCodeGenerator(entModel, cfg);
      var genSource = codeGen.GenerateSource();
      File.WriteAllText(cfg.OutputPath, genSource); //write to file - 

      // compile it
      var compileResult = SourceCodeGenUtil.CompileSources(cfg.Driver.GetType(), genSource);
      foreach (CompilerError err in compileResult.Errors)
        Debug.WriteLine("Error: " + err.ErrorText + " Line: " + err.Line);
      Assert.IsTrue(compileResult.Errors.Count == 0, "Compiler errors found.");

      // Create entity store and verify there are no schema differences
      var connString = SetupHelper.GetConnectionString();
      var modelTypeName = cfg.Namespace + "." + cfg.ModelClassName;
      var modelType = compileResult.CompiledAssembly.GetType(modelTypeName);
      var log = new Logging.Log();
      var nonProcActions = SourceCodeGenUtil.CompareDatabaseSchemas(compileResult.CompiledAssembly, modelType, cfg, SetupHelper.GetDriver(), DbOptions.Default, 
                                                                    connString, log);
      if (nonProcActions.Count > 0)
        foreach (var action in nonProcActions)
          Debug.WriteLine("Schema update: " + action.ToString());
      Assert.IsTrue(nonProcActions.Count == 0, "Schema differences detected.");
    }
  
  }//class
}
