﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;
using System.Data.SqlClient;
using System.Threading;

using Vita.Entities;
using Vita.Entities.Runtime;
using Vita.Modules.ErrorLogging;

using Vita.Samples.BooksModel;

namespace Vita.Samples.BooksSample.UnitTests {

#if USE_NUNIT
  using NUnit.Framework;
  using TestClass = NUnit.Framework.TestFixtureAttribute;
  using TestMethod = NUnit.Framework.TestAttribute;
  using TestInitialize = NUnit.Framework.SetUpAttribute;
  using TestCategory = NUnit.Framework.CategoryAttribute;
#else
  using Microsoft.VisualStudio.TestTools.UnitTesting;
  using Vita.Entities.Caching;
#endif

  // Note: temporarily disabled with move to BLToolkit Linq engine; will recover back later
  [TestClass]
  public class MiscTests {

    [TestInitialize]
    public void TestInit() {
      SetupHelper.Init();
      SetupHelper.CreateSampleData();
    }
    [TestCleanup]
    public void TearDown() {
      SetupHelper.TearDown(); 
    }

#if MSSQL
    [TestMethod, TestCategory("Vita_Books")]
#endif
    public void TestCustomStoredProcs() {
      var entityStore = SetupHelper.EntityStore;
      var booksModule = SetupHelper.BooksModel.MainModule;
      var session = entityStore.OpenSession();
      // Using SELECT custom stored procedures BooksGetByAuthor and BookGetLatestByPublisher. -------------------------------------------
      // Get books by author name
      session = entityStore.OpenSession();
      var booksByJack = booksModule.GetBooksByAuthor(session, "Pound");
      Assert.IsTrue(2 == booksByJack.Count, "Found wrong # of books by Jack Pound.");
      var latestByMs = booksModule.GetLatestBookByPublisher(session, "MS Books");
      Assert.IsNotNull(latestByMs, "Latest book by MS not found.");

      //Note: SQL CE does not support some SQL constructs we use in custom UPDATE/DELETE SQLs
      // Using UPDATE custom stored procedure BooksChangePrice ------------------------------------------
      // Use stored proc to reduce the price by 20% on all kids books. We have Kid book with original price 10
      booksModule.ChangePrice(session, BookCategory.Kids, -20);
      //check that changes actually took place
      session = entityStore.OpenSession();
      var kidBook = session.EntitySet<IBook>().First(b => b.Title.StartsWith("Three")); //Three little pigs
      //Original price 10, now it should be 8
      Assert.IsTrue(Math.Abs(8 - kidBook.Price) < 0.01, "Kids book price is not reduced after discount.");
      //  Done, verified - hardcover book price changed from 10 to 8.

      var kidPubId = kidBook.Publisher.Id;
      // Using DELETE custom stored procedure BooksDeleteByPublishedDate --------------------------------------
      //    Let's add an old book with price < $2 (cut-off price) 
      session = entityStore.OpenSession();
      var msPub = session.EntitySet<IPublisher>().First(p => p.Name.StartsWith("MS"));
      var vbOld = session.NewBook(BookEdition.Paperback | BookEdition.Hardcover, BookCategory.Programming,
                          "VB 6 Programming", "Programming in VB", msPub, DateTime.Now.AddYears(-15), 1.0);
      var vbOldId = vbOld.Id;
      session.SaveChanges();
      //Check it exists
      session = entityStore.OpenSession();
      vbOld = session.GetEntity<IBook>(vbOldId);
      Assert.IsTrue(vbOld != null, "VB6 book not created.");
      // Now delete it using custom method
      session = entityStore.OpenSession();
      booksModule.DeleteOldCheapBooks(session, DateTime.Now.AddYears(-5));
      //check it is deleted
      session = entityStore.OpenSession();
      vbOld = session.GetEntity<IBook>(vbOldId);
      Assert.IsTrue(vbOld == null, "VB6 book not deleted.");

      // Test DiscountOldFictionBooks
      // Create an old fiction book
      session = entityStore.OpenSession();
      var kidPub = session.GetEntity<IPublisher>(kidPubId);
      var histBook = session.NewBook(BookEdition.Paperback, BookCategory.Fiction,
                          "Ancient Rome", "History of ancient Rome.", kidPub, DateTime.Now.AddYears(-15), 15.0);
      session.SaveChanges();
      // Discount all fiction books
      session = entityStore.OpenSession();
      booksModule.DiscountOldFictionBooks(session, new DateTime(2000, 1, 1), 1.0);
      // Validate - histBook should have price $1
      session = entityStore.OpenSession();
      histBook = session.GetEntity<IBook>(histBook.Id);
      Assert.IsTrue(histBook.Price < 1.1, "History book was not discounted.");
    }
  }//class
}
