﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;

using Vita.Entities;

namespace Vita.UnitTests.Identity {
#if USE_NUNIT
  using NUnit.Framework;
  using TestClass = NUnit.Framework.TestFixtureAttribute;
  using TestMethod = NUnit.Framework.TestAttribute;
  using TestInitialize = NUnit.Framework.SetUpAttribute;
  using TestCategory = NUnit.Framework.CategoryAttribute;
#else
  using Microsoft.VisualStudio.TestTools.UnitTesting;
#endif

  [Entity]
  public interface ICar {
    [PrimaryKey, Identity]
    long Id { get; set; }
    [Size(30)]
    string Model { get; set; }
    IPerson Owner { get; set; }
  }

  [Entity]
  public interface IPerson {
    [PrimaryKey, Identity]
    int Id { get; set; }
    [Size(30)]
    string Name { get; set; }
  }

  // We skip defining custom entity module and use base EntityModule class
  public class IdentityTestsEntityModel : EntityModel {
    public IdentityTestsEntityModel() {
      var area = AddArea("ident", "ident");
      var mainModule = new EntityModule(area, "MainModule");
      mainModule.RegisterEntities(typeof(ICar), typeof(IPerson));
    }
  }//class



// SQL CE does not support OUTPUT parameters (used by VITA for db Identity fields) - need to find workaround for this.
// For now we don't run this test in SQL CE
#if !SQLCE
  [TestClass]
#endif
  public class IdentityColumnsTest {

    [TestMethod, TestCategory("Vita_Base")]
    public void TestIdentityColumns() {
      var model = new IdentityTestsEntityModel(); 
      var entStore = SetupHelper.CreateTestEntityStore(model);
      SetupHelper.DeleteAll(entStore, typeof(ICar), typeof(IPerson)); 
      var session = entStore.OpenSession();
      var john = session.NewEntity<IPerson>();
      john.Name = "John S";
      var car1 = session.NewEntity<ICar>();
      car1.Model = "Beatle";
      car1.Owner = john;
      var car2 = session.NewEntity<ICar>();
      car2.Model = "Explorer";
      car2.Owner = john;
      Assert.IsTrue(car1.Id < 0, "Car ID is expected to be < 0 before saving."); 
      session.SaveChanges();
      //Check that Identity values immediately changed to actual positive values loaded from database
      Assert.IsTrue(john.Id > 0, "Invalid person ID - expected positive value.");
      Assert.IsTrue(car1.Id > 0, "Invalid car1 ID - expected positive value.");
      Assert.IsTrue(car2.Id > 0, "Invalid car2 ID - expected positive value."); 

      //Start new session, load all and check that IDs and relationships are correct
      session = entStore.OpenSession();
      var persons = session.GetEntities<IPerson>(take: 100).ToList();
      john = persons[0];
      var cars = session.GetEntities<ICar>(take: 10).ToList();
      car1 = cars[0];
      car2 = cars[1];
      Assert.IsTrue(john.Id > 0 && car1.Id > 0 && car2.Id > 0, "IDs expected to become > 0 after saving.");
      Assert.IsTrue(car1.Owner == john, "Owner expected to be John");
      Assert.IsTrue(car2.Owner == john, "Owner expected to be John");
    }//method
  }
}
