﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

using Vita.Common;
using Vita.Data.Caching;
using Vita.Entities.Runtime;


namespace Vita.UnitTests {
#if USE_NUNIT
  using NUnit.Framework;
  using TestClass = NUnit.Framework.TestFixtureAttribute;
  using TestMethod = NUnit.Framework.TestAttribute;
  using TestInitialize = NUnit.Framework.SetUpAttribute;
  using TestCategory = NUnit.Framework.CategoryAttribute;
#else
  using Microsoft.VisualStudio.TestTools.UnitTesting;
  using System.Threading.Tasks;
  using System.Diagnostics;
#endif

  [TestClass]
  public class TaskHelperTests {

    [TestMethod, TestCategory("Vita_Base")]
    public void TestTaskErrorHandling() {
      // Add handler to TaskHelper.TaskError event, to catch the error in task
      Exception exc = null; 
      TaskHelper.TaskError += (s, e) => { 
        exc = e.Task.Exception;
      };
      var task = TaskHelper.NewTask(AsyncTaskWithError);
      task.Start();
      Thread.Sleep(200);
      Assert.IsNotNull(exc, "Error was not handled.");
      // exc is in fact AggregateException that wraps the original exception(s)
      var aggrExc = exc as AggregateException;
      Assert.IsNotNull(aggrExc, "Exc is not AggregateException.");
      Assert.IsTrue(aggrExc.InnerExceptions.Count == 1, "Invalid # of wrapped exceptions.");
    }

    private void AsyncTaskWithError() {
      throw new Exception("Error thrown in async task.");
    }

    /*  
        //Just an illustration that global error handler does not work with thread pool. If you uncomment and run this test, it will failed on the last check 
        // of _logged variable - indicating that CurrentDomain_UnhandledException was not invoked. 
        bool _error = false;
        bool _logged = false;
        [TestMethod, TestCategory("Vita_Base")]
        public void TestThreadingPoolError() {
          AppDomain.CurrentDomain.UnhandledException += CurrentDomain_UnhandledException;
          ThreadPool.QueueUserWorkItem(ThreadPoolMethodWithError);
          Thread.Sleep(500);
          AppDomain.CurrentDomain.UnhandledException -= CurrentDomain_UnhandledException;
          Assert.IsTrue(_error, "error not thrown.");
          Assert.IsTrue(_logged, "error not logged.");
        }

        private void ThreadPoolMethodWithError(object state) {
          _error = true;
          throw new Exception("Error thrown in thread pool method.");
        }
        void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e) {
          _logged = true;
          Debug.WriteLine(" Caught in domain.UnhandledException! " + e.ExceptionObject);
          if (e.IsTerminating)
            Debug.WriteLine("terminating!!!!");
        }
     */




  } //class 
}
